module HCOUP_SM
        use SM_const
	use pv
	use Gauge_boson_2p
        use SM_Hdecay
        use func_forQCDco
        use spence_func
        use int_romberg
	implicit none

	contains
!-===============  FUNCTIONS ============================
!##########################################################
!
!    SECTION I Tree-level couplings    
!
!##########################################################

! These lambda couplings are defined as the coefficients in Lagrangian. 

      double precision function lam_hhh_sm(mh)
	double precision, intent(in):: mh

	lam_hhh_sm = -mh**2/(2.d0*vev)

	end function
!--------------------------------------------
        double precision function lam_GpGmh_sm(mh)
	double precision, intent(in):: mh

	lam_GpGmh_sm = -mh**2/vev

	 end function 
!--------------------------------------------
      double precision function lam_GGh_sm(mh)
	double precision, intent(in):: mh

	lam_GGh_sm = -mh**2/(2.d0*vev) 

	 end function 
!--------------------------------------------
      double precision function lam_hhhh_sm(mh)
	double precision, intent(in):: mh

	lam_hhhh_sm = -mh**2/(8.d0*vev**2)

	 end function 
!--------------------------------------------
      double precision function lam_GpGmhh_sm(mh)
	double precision, intent(in):: mh

	lam_GpGmhh_sm = -mh**2/(2.d0*vev**2)

	 end function 
!--------------------------------------------
      double precision function lam_GGhh_sm(mh)
	double precision, intent(in):: mh

	lam_GGhh_sm = -mh**2/(4.d0*vev**2)

	 end function 
!##########################################################
!
!    SECTION II IPI DIAGRAMS
!
!##########################################################
!====== 1-point functions ========
	complex (kind(0d0)) function Th_fermion(mf)
	double precision, intent(in):: mf

	Th_fermion = -4.d0*mf**2/vev/(16.d0*pi**2)*a0f(mf)

	 end function
!--------------------------------------------------------
	complex (kind(0d0)) function Th_F_SM()

        Th_F_SM = nc*Th_fermion(mt)&
                 +nc*Th_fermion(mb)&
                 +nc*Th_fermion(mc)&
                 +Th_fermion(mtau)

	 end function
!---------------------------------------------
	complex (kind(0d0)) function Th_B_SM()
	double precision x

	Th_B_SM =  1.d0/(16.d0*pi**2)*&
	       (&	
	        -3.d0*lam_hhh_sm(mh)*a0f(mh)& 
		-lam_GpGmh_sm(mh)*a0f(mw)-lam_GGh_sm(mh)*a0f(mz)&
	        +3.d0*g*mW*a0f(mW)+3.d0/2.d0*gz*mZ*a0f(mZ)-2.d0*g*mW**3-gz*mZ**3&
		)

	 end function
!====== 2-point functions ========

!====== Higgs boson two point functions ========
	complex (kind(0d0)) function PiGG_fermion(qsq,mf)
	double precision, intent(in):: qsq,mf

	PiGG_fermion=&
	 -4.d0*mf**2/(16.d0*pi**2*vev**2)*&
	( a0f(mf) -qsq/2.d0*b0f(qsq,mf,mf) )

	 end function
!----------------------------------------------------
	complex (kind(0d0)) function Pihh_fermion(qsq,mf)
	double precision, intent(in):: qsq,mf

	Pihh_fermion=&
	 -4.d0*mf**2/(16.d0*pi**2*vev**2)*&
	( a0f(mf) + (2.d0*mf**2-qsq/2.d0)*b0f(qsq,mf,mf) )

	 end function
!-----------------------------------------------------
	complex (kind(0d0)) function dPihh_fermion(qsq,mf)
	double precision, intent(in):: qsq,mf

	dPihh_fermion=&
	 -4.d0*mf**2/(16.d0*pi**2*vev**2)*&
	( (2.d0*mf**2-qsq/2.d0)*db0f(qsq,mf,mf)-1.d0/2.d0*b0f(qsq,mf,mf) )

	 end function
!=======================================================
	complex (kind(0d0)) function Pihh_F_SM(qsq)
	double precision, intent(in):: qsq

	Pihh_F_SM= nc*Pihh_fermion(qsq,mt)&
                    +nc*Pihh_fermion(qsq,mc)&
                    +nc*Pihh_fermion(qsq,mb)&
                    +Pihh_fermion(qsq,mtau)
	
	 end function
!---------------------------------------------
	complex (kind(0d0)) function Pihh_B_SM(qsq)
	double precision, intent(in):: qsq

	Pihh_B_SM=&
	 1.d0/(16.d0*pi**2)*&
	(&
	  g**2*(3.d0*mw**2-qsq)*b0f(qsq,mW,mW)&
	  +g**2/2.d0*(3.d0)*a0f(mW)&
	  +gz**2/2.d0*(3.d0*mz**2-qsq)*b0f(qsq,mZ,mZ)&
	  +gz**2/4.d0*(3.d0)*a0f(mZ)&
	  -(3.d0/2.d0)*(2.d0*g**2*mW**2+gz**2*mZ**2)&
	)&
!
	+1.d0/(16.d0*pi**2)*&
	( -12.d0*lam_hhhh_sm(mh)*a0f(mh)&
	 -2.d0*lam_GpGmhh_sm(mh)*a0f(mw)-2.d0*lam_GGhh_sm(mh)*a0f(mz)&
	 +lam_GpGmh_sm(mh)**2*b0f(qsq,mw,mw)&
	 +2.d0*lam_GGh_sm(mh)**2*b0f(qsq,mz,mz)+18.d0*lam_hhh_sm(mh)**2*b0f(qsq,mh,mh)&
	 	)
	
	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function dPihh_F_SM(qsq)
	double precision, intent(in):: qsq

	dPihh_F_SM = nc*dPihh_fermion(qsq,mt)&
                    +nc*dPihh_fermion(qsq,mc)&
                    +nc*dPihh_fermion(qsq,mb)&
                    +dPihh_fermion(qsq,mtau)
	
	 end function
!---------------------------------------------
	complex (kind(0d0)) function dPihh_B_SM(qsq)
	double precision, intent(in):: qsq

	dPihh_B_SM=&
	 1.d0/(16.d0*pi**2)*&
	(&
	  g**2*(3.d0*mw**2-qsq)*db0f(qsq,mW,mW)&
	  +g**2*(-1.d0)*b0f(qsq,mW,mW)&
	  +gz**2/2.d0*(3.d0*mz**2-qsq)*db0f(qsq,mZ,mZ)&
	  +gz**2/2.d0*(-1.d0)*b0f(qsq,mZ,mZ)&
	)&
!
	+1.d0/(16.d0*pi**2)*&
	(&
	 +lam_GpGmh_SM(mh)**2*db0f(qsq,mw,mw)&
	 +2.d0*lam_GGh_SM(mh)**2*db0f(qsq,mz,mz)&
	 +18.d0*lam_hhh_SM(mh)**2*db0f(qsq,mh,mh)&
	)
	
	 end function
!======= fermion 2-point functions ========================
	complex (kind(0d0)) function Piff_S_SM(qsq,mf)
	double precision, intent(in):: qsq,mf
	double precision:: vf,af,qf,mfp
        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	Piff_S_SM = -1.d0/(16.d0*pi**2)*(&
	QED*2.d0*(g*sw)**2*Qf**2*(2.d0*b0f(qsq,mf,mphoton)-1.d0)&
	+2.d0*gz**2*(vf**2-af**2)*(2.d0*b0f(qsq,mf,mZ)-1.d0)&
	+mf**2/vev**2*(b0f(qsq,mf,mz)-b0f(qsq,mf,mh)&
	+2.d0*mfp**2/mf**2*b0f(qsq,mfp,mw)&	
	            )&
	)
	
	 end function
!---------------------------------------------
	complex (kind(0d0)) function dPiff_S_SM(qsq,mf)
	double precision, intent(in):: qsq,mf
	double precision:: vf,af,qf,mfp
        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	dPiff_S_SM = -1.d0/(16.d0*pi**2)*(&
	QED*2.d0*(g*sw)**2*Qf**2*(2.d0*db0f(qsq,mf,mphoton))&
	+2.d0*gz**2*(vf**2-af**2)*(2.d0*db0f(qsq,mf,mZ))&
	+mf**2/vev**2*(db0f(qsq,mf,mz)-db0f(qsq,mf,mh)&
	+2.d0*mfp**2/mf**2*db0f(qsq,mfp,mw)&	
	            )&
	)
	
	 end function
!---------------------------------------------
	complex (kind(0d0)) function Piff_V_SM(qsq,mf)
	double precision, intent(in):: qsq,mf
	double precision:: vf,af,qf,mfp
        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	Piff_V_SM = -1.d0/(16.d0*pi**2)*(&
	QED*(g*sw)**2*Qf**2*(2.d0*b1f(qsq,mf,mphoton)+1.d0)&
	+gz**2*(vf**2+af**2)*(2.d0*b1f(qsq,mf,mZ)+1.d0)&
	+g**2/4.d0*(2.d0*b1f(qsq,mfp,mW)+1.d0)&
	+mf**2/vev**2*(b1f(qsq,mf,mh)+b1f(qsq,mf,mz)&
	+(mfp**2+mf**2)/mf**2*b1f(qsq,mfp,mw)&	
	            )&
	)
	
	 end function
!---------------------------------------------
	complex (kind(0d0)) function dPiff_V_SM(qsq,mf)
	double precision, intent(in):: qsq,mf
	double precision:: vf,af,qf,mfp
        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	dPiff_V_SM = -1.d0/(16.d0*pi**2)*(&
	QED*(g*sw)**2*Qf**2*(2.d0*db1f(qsq,mf,mphoton))&
	+gz**2*(vf**2+af**2)*(2.d0*db1f(qsq,mf,mZ))&
	+g**2/4.d0*(2.d0*db1f(qsq,mfp,mW))&
	+mf**2/vev**2*(db1f(qsq,mf,mh)+db1f(qsq,mf,mz)&
	+(mfp**2+mf**2)/mf**2*db1f(qsq,mfp,mw)&	
	            )&
	)
	
	 end function
!====== Gauge boson two point functions ========
	complex (kind(0d0)) function PiWW_B_SM(qsq)
	double precision, intent(in):: qsq
	
	PiWW_B_SM=PiWW_B_WOH(qsq)+g**2/(16.d0*pi**2)*( mw**2*b0f(qsq,mh,mW)+1.d0/4.d0*b5f(qsq,mh,mw) )

 	end function
!--------------------------------------------- 
	complex (kind(0d0)) function dPiWW_B_SM(qsq)
	double precision, intent(in):: qsq


	dPiWW_B_SM=dPiWW_B_WOH(qsq)+ g**2/(16.d0*pi**2)*( mW**2*db0f(qsq,mh,mW)+1.d0/4.d0*db5f(qsq,mh,mW) )

 	end function
!---------------------------------------------
	complex (kind(0d0)) function PiZZ_B_SM(qsq)
	double precision, intent(in):: qsq
	
	PiZZ_B_SM=PiZZ_B_WOH(qsq)+ g**2/cw**2/(16.d0*pi**2)*( mZ**2*b0f(qsq,mh,mZ)+1.d0/4.d0*b5f(qsq,mh,mZ) )

 	end function
!--------------------------------------------- added from v2
	complex (kind(0d0)) function dPiZZ_B_SM(qsq)
	double precision, intent(in):: qsq
	
	dPiZZ_B_SM=dPiZZ_B_WOH(qsq)+ g**2/cw**2/(16.d0*pi**2)*( mZ**2*db0f(qsq,mh,mZ)+1.d0/4.d0*db5f(qsq,mh,mZ) )

 	end function
!---------------------------------------------
	complex (kind(0d0)) function PiGG_B_SM(qsq)
	double precision, intent(in):: qsq
	
	PiGG_B_SM = PiGG_B_WOH(qsq)

 	end function
!---------------------------------------------
	complex (kind(0d0)) function PiZG_B_SM(qsq)
	double precision, intent(in):: qsq
	
	PiZG_B_SM = PiZG_B_WOH(qsq)

 	end function
!---------------------------------------------
	complex (kind(0d0)) function dPiGG_B_SM(qsq)
	double precision, intent(in):: qsq
	
	dPiGG_B_SM = dPiGG_B_WOH(qsq)

 	end function
!============== 3-point functions  ================================
        complex (kind(0d0)) function Gam_hhh_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	double precision:: p12
	p12 = 1.d0/2.d0*(qsq-p1sq-p2sq)

	Gam_hhh_F_SM = &
      -8.d0*mt**4*3.d0/(16.d0*pi**2)/vev**3*&
      (&
       b0f(p1sq,mt,mt)+b0f(p2sq,mt,mt)+b0f(qsq,mt,mt)&
      +(&
         ( 4.d0*mt**2-qsq+(qsq-p1sq-p2sq)/2.d0 )*c0f(p1sq,p2sq,qsq,mt,mt,mt)&
        )&
      )&
! 
      -8.d0*mc**4*3.d0/(16.d0*pi**2)/vev**3*&
      (&
       b0f(p1sq,mc,mc)+b0f(p2sq,mc,mc)+b0f(qsq,mc,mc)&
      +(&
         ( 4.d0*mc**2-qsq+(qsq-p1sq-p2sq)/2.d0 )*c0f(p1sq,p2sq,qsq,mc,mc,mc)&
        )&
      )&
!
      -8.d0*mb**4*3.d0/(16.d0*pi**2)/vev**3*&
      (&
       b0f(p1sq,mb,mb)+b0f(p2sq,mb,mb)+b0f(qsq,mb,mb)&
      +(&
         ( 4.d0*mb**2-qsq+(qsq-p1sq-p2sq)/2.d0 )*c0f(p1sq,p2sq,qsq,mb,mb,mb)&
      )&
      )&
! 
      -8.d0*mtau**4/(16.d0*pi**2)/vev**3*&
      (&
       b0f(p1sq,mtau,mtau)+b0f(p2sq,mtau,mtau)+b0f(qsq,mtau,mtau)&
      +(&
         ( 4.d0*mtau**2-qsq+(qsq-p1sq-p2sq)/2.d0 )*c0f(p1sq,p2sq,qsq,mtau,mtau,mtau)&
      )&
      )
	
	 end function
!---------------------------------------------
	complex (kind(0d0)) function Gam_hhh_B_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam_hhh_B_SM =&
       1.d0/(16.d0*pi**2)*(&
!~~~~~~~~~~~ 3-Gauge ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
       +8.d0*g**3*mw**3*c0f(p1sq,p2sq,qsq,mW,mW,mW)&
       -1.d0/2.d0*g**3*mw**3*c0f(p1sq,p2sq,qsq,mw,mw,mw)&
       +4.d0*gz**3*mz**3*c0f(p1sq,p2sq,qsq,mZ,mZ,mZ)&
       -1.d0/4.d0*gz**3*mz**3*c0f(p1sq,p2sq,qsq,mz,mz,mz)&
!~~~~~~~~~~~ 2-Gauge ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
      -1.d0/2.d0*g**3*mw*Chhh_SVV(p1sq,p2sq,qsq,mw,mW,mW)&
      -1.d0/4.d0*gz**3*mz*Chhh_SVV(p1sq,p2sq,qsq,mz,mZ,mZ)&
!~~~~~~~~~~~ 1-Gauge ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
      +1.d0/2.d0*g**2*lam_GpGmh_sm(mh)*Chhh_VSS(p1sq,p2sq,qsq,mW,mw,mw)&
      +1.d0/2.d0*gz**2*lam_GGh_sm(mh)*Chhh_VSS(p1sq,p2sq,qsq,mZ,mz,mz)&
!~~~~~~~~~~~ 0-Gauge ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
      -2.d0*lam_GpGmh_sm(mh)**3*c0f(p1sq,p2sq,qsq,mw,mw,mw)&
      -8.d0*lam_GGh_sm(mh)**3*c0f(p1sq,p2sq,qsq,mz,mz,mz)&
      -216.d0*lam_hhh_sm(mh)**3*c0f(p1sq,p2sq,qsq,mh,mh,mh)&
!~~~~~~~~~~~ B function ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
      +2.d0*g**3*mw*(b0f(p1sq,mW,mW)+b0f(p2sq,mW,mW)+b0f(qsq,mW,mW))&        !! qsq can be separated into p1sq and p2sq and qsq
      +gz**3*mz*(b0f(p1sq,mZ,mZ)+b0f(p2sq,mZ,mZ)+b0f(qsq,mZ,mZ))&
      +2.d0*lam_GpGmh_sm(mh)*lam_GpGmhh_sm(mh)*&
        (b0f(p1sq,mw,mw)+b0f(p2sq,mw,mw)+b0f(qsq,mw,mw))&
      +4.d0*lam_GGh_sm(mh)*lam_GGhh_sm(mh)*&
        (b0f(p1sq,mz,mz)+b0f(p2sq,mz,mz)+b0f(qsq,mz,mz))&
      +72.d0*lam_hhh_sm(mh)*lam_hhhh_sm(mh)*&
        (b0f(p1sq,mh,mh)+b0f(p2sq,mh,mh)+b0f(qsq,mh,mh))&
     )&
	-1.d0/(16.d0*pi**2)*(3.d0*g**3*mW+3.d0/2.d0*gz**3*mZ)

	 end function
!===================================================================
	complex (kind(0d0)) function Gam_hff_S_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	Gam_hff_S_SM = 1.d0/(16.d0*pi**2)*mf/vev*(&
	-4.d0*gz**2*(vf**2-af**2)*Chff_FVF(p1sq,p2sq,qsq,mf,mz)&
	-QED*4.d0*(g*sw)**2*Qf**2*Chff_FVF(p1sq,p2sq,qsq,mf,mphoton)&
	+mf**2/vev**2*Chff_FSF(p1sq,p2sq,qsq,mf,mh)&
	-mf**2/vev**2*Chff_FSF(p1sq,p2sq,qsq,mf,mz)&
	-2.d0*mfp**2/vev**2*Chff_FSF(p1sq,p2sq,qsq,mfp,mw)&
	-2.d0*gz**4*vev**2*(vf**2-af**2)*c0f(p1sq,p2sq,qsq,mZ,mf,mZ)&
	-6.d0*lam_hhh_sm(mh)*mf**2/vev*c0f(p1sq,p2sq,qsq,mh,mf,mh)&
	+2.d0*lam_GGh_sm(mh)*mf**2/vev*c0f(p1sq,p2sq,qsq,mz,mf,mz)&
	+2.d0*lam_GpGmh_sm(mh)*mfp**2/vev*c0f(p1sq,p2sq,qsq,mw,mfp,mw)&
	-g**2/4.d0*Chff_VFS(p1sq,p2sq,qsq,mW,mfp,mw)&
	-g**2/4.d0*Chff_SFV(p1sq,p2sq,qsq,mw,mfp,mW)&
	-gz**2/8.d0*Chff_VFS(p1sq,p2sq,qsq,mZ,mf,mz)&
	-gz**2/8.d0*Chff_SFV(p1sq,p2sq,qsq,mz,mf,mZ)&
	)

	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_P_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_P_SM = -1.d0/(16.d0*pi**2)*mf/vev*(& 
           -g**2/4.d0*(Chff_VFS(p1sq,p2sq,qsq,mW,mfp,mw)&
                      -Chff_SFV(p1sq,p2sq,qsq,mw,mfp,mW))&
           -2.d0*gz**2*vf*af*(Chff_VFS(p1sq,p2sq,qsq,mZ,mf,mz)&
                      -Chff_SFV(p1sq,p2sq,qsq,mz,mf,mZ))&              
                                       )

	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_V1_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_V1_SM = -1.d0/(16.d0*pi**2)*(& 
             2.d0*gz**2*(vf**2 + af**2)*mf**2/vev*(c0f(p1sq,p2sq,qsq,mf,mZ,mf)&
                                              +2.d0*c11f(p1sq,p2sq,qsq,mf,mZ,mf))&
            +QED*2.d0*(g*sw)**2*Qf**2*mf**2/vev*(c0f(p1sq,p2sq,qsq,mf,mphoton,mf)&
                                      +2.d0*c11f(p1sq,p2sq,qsq,mf,mphoton,mf))&
            +g**2/2.d0*mfp**2/vev*(c0f(p1sq,p2sq,qsq,mfp,mW,mfp)&
                            +2.d0*c11f(p1sq,p2sq,qsq,mfp,mW,mfp))&
            +mf**4/vev**3*(c0f(p1sq,p2sq,qsq,mf,mh,mf)&
                            +2.d0*c11f(p1sq,p2sq,qsq,mf,mh,mf))&
            +mf**4/vev**3*(c0f(p1sq,p2sq,qsq,mf,mz,mf)&
                            +2.d0*c11f(p1sq,p2sq,qsq,mf,mz,mf))&
            +mfp**2/vev**3*(mf**2+mfp**2)*(c0f(p1sq,p2sq,qsq,mfp,mw,mfp)&
                             +2.d0*c11f(p1sq,p2sq,qsq,mfp,mw,mfp))&
            -gz**4*vev*(vf**2 + af**2)*(c0f(p1sq,p2sq,qsq,mZ,mf,mZ)&
                                      +c11f(p1sq,p2sq,qsq,mZ,mf,mZ))&
            -g**4/4.d0*vev*(c0f(p1sq,p2sq,qsq,mW,mfp,mW)&
                           +c11f(p1sq,p2sq,qsq,mW,mfp,mW))&
             +6.d0*lam_hhh_sm(mh)*mf**2/vev**2*(c0f(p1sq,p2sq,qsq,mh,mf,mh)&
                            +c11f(p1sq,p2sq,qsq,mh,mf,mh))&
             +2.d0*lam_GGh_sm(mh)*mf**2/vev**2*(c0f(p1sq,p2sq,qsq,mz,mf,mz)&
                            +c11f(p1sq,p2sq,qsq,mz,mf,mz))&
             +1.d0*lam_GpGmh_sm(mh)*(mf**2+mfp**2)/vev**2*(c0f(p1sq,p2sq,qsq,mw,mfp,mw)&
                            +c11f(p1sq,p2sq,qsq,mw,mfp,mw))&
             -g**2/4.d0*mfp**2/vev*(2.d0*c0f(p1sq,p2sq,qsq,mW,mfp,mw)&
                                        +c11f(p1sq,p2sq,qsq,mw,mfp,mw)&
                                   -c0f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                   +c11f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                   )&
             -gz**2/8.d0*mf**2/vev*(2.d0*c0f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                        +c11f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                   -c0f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                   +c11f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                   )&
                                       )

	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_V2_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_V2_SM = -1.d0/(16.d0*pi**2)*(& 
             2.d0*gz**2*(vf**2 + af**2)*mf**2/vev*(c0f(p1sq,p2sq,qsq,mf,mZ,mf)&
                                              +2.d0*c12f(p1sq,p2sq,qsq,mf,mZ,mf))&
            +QED*2.d0*(g*sw)**2*Qf**2*mf**2/vev*(c0f(p1sq,p2sq,qsq,mf,mphoton,mf)&
                                      +2.d0*c12f(p1sq,p2sq,qsq,mf,mphoton,mf))&
            +g**2/2.d0*mfp**2/vev*(c0f(p1sq,p2sq,qsq,mfp,mW,mfp)&
                            +2.d0*c12f(p1sq,p2sq,qsq,mfp,mW,mfp))&
            +mf**4/vev**3*(c0f(p1sq,p2sq,qsq,mf,mh,mf)&
                            +2.d0*c12f(p1sq,p2sq,qsq,mf,mh,mf))&
            +mf**4/vev**3*(c0f(p1sq,p2sq,qsq,mf,mz,mf)&
                            +2.d0*c12f(p1sq,p2sq,qsq,mf,mz,mf))&
            +mfp**2/vev**3*(mf**2+mfp**2)*(c0f(p1sq,p2sq,qsq,mfp,mw,mfp)&
                            +2.d0*c12f(p1sq,p2sq,qsq,mfp,mw,mfp))&
            -gz**4*vev*(vf**2 + af**2)*(c12f(p1sq,p2sq,qsq,mZ,mf,mZ))&
            -g**4/4.d0*vev*(c12f(p1sq,p2sq,qsq,mW,mfp,mW))&
            +6.d0*lam_hhh_sm(mh)*mf**2/vev**2*(c12f(p1sq,p2sq,qsq,mh,mf,mh))&
            +2.d0*lam_GGh_sm(mh)*mf**2/vev**2*(c12f(p1sq,p2sq,qsq,mz,mf,mz))&
            +1.d0*lam_GpGmh_sm(mh)*(mf**2+mfp**2)/vev**2*(c12f(p1sq,p2sq,qsq,mw,mfp,mw))&
            -g**2/4.d0*mfp**2/vev*(2.d0*c0f(p1sq,p2sq,qsq,mW,mfp,mw)&
                                       +c12f(p1sq,p2sq,qsq,mw,mfp,mw)&
                                  -c0f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                  +c12f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                  )&
            -gz**2/8.d0*mf**2/vev*(2.d0*c0f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                       +c12f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                  -c0f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                  +c12f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                  )&
                                       )

	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_A1_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_A1_SM = -1.d0/(16.d0*pi**2)*(& 
             -4.d0*gz**2*vf*af*mf**2/vev*(c0f(p1sq,p2sq,qsq,mf,mZ,mf)&
                                    +2.d0*c11f(p1sq,p2sq,qsq,mf,mZ,mf))&   
             -g**2/2.d0*mfp**2/vev*(c0f(p1sq,p2sq,qsq,mfp,mW,mfp)&
                                    +2.d0*c11f(p1sq,p2sq,qsq,mfp,mW,mfp))&
              +mfp**2*(mf**2 - mfp**2)/vev**3*(c0f(p1sq,p2sq,qsq,mfp,mw,mfp)&
                                     +2.d0*c11f(p1sq,p2sq,qsq,mfp,mw,mfp))&
              +2.d0*gz**4*vf*af*vev*(c0f(p1sq,p2sq,qsq,mZ,mf,mZ)&
                                    +c11f(p1sq,p2sq,qsq,mZ,mf,mZ))&
              +g**4/4.d0*vev*(c0f(p1sq,p2sq,qsq,mW,mfp,mW)&
                                    +c11f(p1sq,p2sq,qsq,mW,mfp,mW))&
              +lam_GpGmh_sm(mh)*(mf**2-mfp**2)/vev**2*(c0f(p1sq,p2sq,qsq,mw,mfp,mw)&
                                    +c11f(p1sq,p2sq,qsq,mw,mfp,mw))&
             +g**2*mfp**2/(4.d0*vev)*(2.d0*c0f(p1sq,p2sq,qsq,mW,mfp,mw)&
                                      +c11f(p1sq,p2sq,qsq,mW,mfp,mw)&
                                      -c0f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                       +c11f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                      )&
             +2.d0*gz**2*vf*af*mf**2/vev*(2.d0*c0f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                      +c11f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                      -c0f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                       +c11f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                      )&
         )

	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_A2_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_A2_SM = -1.d0/(16.d0*pi**2)*(& 
             -4.d0*gz**2*vf*af*mf**2/vev*(c0f(p1sq,p2sq,qsq,mf,mZ,mf)&
                                    +2.d0*c12f(p1sq,p2sq,qsq,mf,mZ,mf))&   
             -g**2/2.d0*mfp**2/vev*(c0f(p1sq,p2sq,qsq,mfp,mW,mfp)&
                                    +2.d0*c12f(p1sq,p2sq,qsq,mfp,mW,mfp))&
             +mfp**2*(mf**2- mfp**2)/vev**3*(c0f(p1sq,p2sq,qsq,mfp,mw,mfp)&
                                    +2.d0*c12f(p1sq,p2sq,qsq,mfp,mw,mfp))&
             +2.d0*gz**4*vf*af*vev*(c12f(p1sq,p2sq,qsq,mZ,mf,mZ))&
             +g**4/4.d0*vev*(c12f(p1sq,p2sq,qsq,mW,mfp,mW))&
             +lam_GpGmh_sm(mh)*(mf**2-mfp**2)/vev**2*(c12f(p1sq,p2sq,qsq,mw,mfp,mw))&
             +g**2*mfp**2/(4.d0*vev)*(2.d0*c0f(p1sq,p2sq,qsq,mW,mfp,mw)&
                                      +c12f(p1sq,p2sq,qsq,mW,mfp,mw)&
                                      -c0f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                       +c12f(p1sq,p2sq,qsq,mw,mfp,mW)&
                                      )&
             +2.d0*gz**2*vf*af*mf**2/vev*(2.d0*c0f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                      +c12f(p1sq,p2sq,qsq,mZ,mf,mz)&
                                      -c0f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                       +c12f(p1sq,p2sq,qsq,mz,mf,mZ)&
                                      )&
         )

	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_T_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_T_SM = -1.d0/(16.d0*pi**2)*(mf/vev)*(&
            mf**2/vev**2*(c11f(p1sq,p2sq,qsq,mf,mh,mf)-c12f(p1sq,p2sq,qsq,mf,mh,mf))&
           -mf**2/vev**2*(c11f(p1sq,p2sq,qsq,mf,mz,mf)-c12f(p1sq,p2sq,qsq,mf,mz,mf))&
           -2.d0*mfp**2/vev**2*(c11f(p1sq,p2sq,qsq,mfp,mw,mfp)-c12f(p1sq,p2sq,qsq,mfp,mw,mfp))&
           +g**2/4.d0*(2.d0*c0f(p1sq,p2sq,qsq,mW,mfp,mw)&
                      +2.d0*c11f(p1sq,p2sq,qsq,mW,mfp,mw)&
                      -1.d0*c12f(p1sq,p2sq,qsq,mW,mfp,mw)&
                      +1.d0*c0f(p1sq,p2sq,qsq,mw,mfp,mW)&
                      +1.d0*c11f(p1sq,p2sq,qsq,mw,mfp,mW)&
                      -2.d0*c12f(p1sq,p2sq,qsq,mw,mfp,mW) )&
           +gz**2/8.d0*(2.d0*c0f(p1sq,p2sq,qsq,mZ,mf,mz)&
                      +2.d0*c11f(p1sq,p2sq,qsq,mZ,mf,mz)&
                      -1.d0*c12f(p1sq,p2sq,qsq,mZ,mf,mz)&
                      +1.d0*c0f(p1sq,p2sq,qsq,mz,mf,mZ)&
                      +1.d0*c11f(p1sq,p2sq,qsq,mz,mf,mZ)&
                      -2.d0*c12f(p1sq,p2sq,qsq,mz,mf,mZ)&
                      )&
                      )
 
	 end function
!------------------------------------------------------------
	complex (kind(0d0)) function Gam_hff_PT_SM(p1sq,p2sq,qsq,mf)
	double precision, intent(in):: p1sq,p2sq,qsq,mf
	double precision:: vf,af,qf,mfp

        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

    Gam_hff_PT_SM = 1.d0/(16.d0*pi**2)*(mf/vev)*(&
           +g**2/4.d0*(2.d0*c0f(p1sq,p2sq,qsq,mW,mfp,mw)&
                      +2.d0*c11f(p1sq,p2sq,qsq,mW,mfp,mw)&
                      -1.d0*c12f(p1sq,p2sq,qsq,mW,mfp,mw)&
                      -1.d0*c0f(p1sq,p2sq,qsq,mw,mfp,mW)&
                      -1.d0*c11f(p1sq,p2sq,qsq,mw,mfp,mW)&
                      +2.d0*c12f(p1sq,p2sq,qsq,mw,mfp,mW) )&
           +2.d0*gz**2*af*vf*(2.d0*c0f(p1sq,p2sq,qsq,mZ,mf,mz)&
                      +2.d0*c11f(p1sq,p2sq,qsq,mZ,mf,mz)&
                      -1.d0*c12f(p1sq,p2sq,qsq,mZ,mf,mz)&
                      -1.d0*c0f(p1sq,p2sq,qsq,mz,mf,mZ)&
                      -1.d0*c11f(p1sq,p2sq,qsq,mz,mf,mZ)&
                      +2.d0*c12f(p1sq,p2sq,qsq,mz,mf,mZ)&
                      )&
                      )
 
	 end function
!==============================================================
 	complex (kind(0d0)) function Gam1_hZZ_fermion(p1sq,p2sq,qsq,mf)
 	double precision, intent(in):: p1sq,p2sq,qsq,mf
 	double precision:: vf,af

       if (mf .eq. mt) then 
        vf = vt ; af = at
       elseif (mf .eq. mc) then 
        vf = vt ; af = at
       elseif (mf .eq. mb) then 
        vf = vb ; af = ab
       elseif (mf .eq. mtau) then 
        vf = ve ; af = ae
       endif

	Gam1_hZZ_fermion = &
	 -32.d0*mf**2*mz**2/vev**3*&
	 1.d0/(16.d0*pi**2)*&
	 (&
	 -1.d0/2.d0*(vf**2 + af**2)*&
	 (&
	 b0f(p1sq,mf,mf)+b0f(p2sq,mf,mf)+2.d0*b0f(qsq,mf,mf)&
	 +(4.d0*mf**2-p1sq-p2sq)*c0f(p1sq,p2sq,qsq,mf,mf,mf)&
	 -8.d0*c24f(p1sq,p2sq,qsq,mf,mf,mf)&
	 )&
	 +1.d0/2.d0*(vf**2-af**2)*&
	   (&
	 +b0f(p2sq,mf,mf)+b0f(p1sq,mf,mf)&
	 +(4.d0*mf**2-qsq)*c0f(p1sq,p2sq,qsq,mf,mf,mf)&
	   )&
	 )

 	 end function
!------------------------------------------------------------------
 	complex (kind(0d0)) function Gam2_hZZ_fermion(p1sq,p2sq,qsq,mf)
 	double precision, intent(in):: p1sq,p2sq,qsq,mf
 	double precision:: vf,af

       if (mf .eq. mt) then 
        vf = vt ; af = at
       elseif (mf .eq. mc) then 
        vf = vt ; af = at
       elseif (mf .eq. mb) then 
        vf = vb ; af = ab
       elseif (mf .eq. mtau) then 
        vf = ve ; af = ae
       endif

	Gam2_hZZ_fermion = &
	 -32.d0*mf**2*mz**4/vev**3/(16.d0*pi**2)*&
	 (&
	 (vf**2 + af**2)*&
	 (4.d0*c23f(p1sq,p2sq,qsq,mf,mf,mf)&
          +3.d0*c12f(p1sq,p2sq,qsq,mf,mf,mf)&
          +c11f(p1sq,p2sq,qsq,mf,mf,mf)&
          +c0f(p1sq,p2sq,qsq,mf,mf,mf)&
          )&
         +(vf**2-af**2)*&
         (c12f(p1sq,p2sq,qsq,mf,mf,mf)&
          -c11f(p1sq,p2sq,qsq,mf,mf,mf)&
          )&
	 )

 	 end function
!----------------------------------------------------------------
 	complex (kind(0d0)) function Gam3_hZZ_fermion(p1sq,p2sq,qsq,mf)
 	double precision, intent(in):: p1sq,p2sq,qsq,mf
 	double precision:: vf,af

       if (mf .eq. mt) then 
        vf = vt ; af = at
       elseif (mf .eq. mc) then 
        vf = vt ; af = at
       elseif (mf .eq. mb) then 
        vf = vb ; af = ab
       elseif (mf .eq. mtau) then 
        vf = ve ; af = ae
       endif

	Gam3_hZZ_fermion = 64.d0*mf**2*mz**4/vev**3/(16.d0*pi**2)*af*vf*&
         (c11f(p1sq,p2sq,qsq,mf,mf,mf)+c12f(p1sq,p2sq,qsq,mf,mf,mf)+c0f(p1sq,p2sq,qsq,mf,mf,mf)&
         )

 	 end function
!-------------------------------------------------------------------
 	complex (kind(0d0)) function Gam1_hWW_fermion(p1sq,p2sq,qsq,mf)
 	double precision, intent(in):: p1sq,p2sq,qsq,mf
 	double precision:: mfp

       if (mf .eq. mt) mfp = mb
       if (mf .eq. mb) mfp = mt
       if (mf .eq. mc) mfp = ms
       if (mf .eq. ms) mfp = mc
       if (mf .eq. mtau) mfp = 0.d0

 	 Gam1_hWW_fermion =4.d0*mw**2*mf**2/vev**3/&
 	 (16.d0*pi**2)*&
 	 (&
 	 1.d0/2.d0*b0f(p2sq,mf,mfp)+b0f(qsq,mf,mf)&
 	 +1.d0/2.d0*b0f(p1sq,mf,mfp)&
 	 -4.d0*c24f(p1sq,p2sq,qsq,mf,mfp,mf)&
 	 +(mf**2+mfp**2-p1sq/2.d0-p2sq/2.d0)*c0f(p1sq,p2sq,qsq,mf,mfp,mf)&
 	 )

 	 end function
!--------------------------------------------
 	complex (kind(0d0)) function Gam2_hWW_fermion(p1sq,p2sq,qsq,mf)
 	double precision, intent(in):: p1sq,p2sq,qsq,mf
 	double precision:: mfp

       if (mf .eq. mt) mfp = mb
       if (mf .eq. mb) mfp = mt
       if (mf .eq. mc) mfp = ms
       if (mf .eq. ms) mfp = mc
       if (mf .eq. mtau) mfp = 0.d0

	Gam2_hWW_fermion =&
	 -4.d0*mw**4*mf**2/vev**3/(16.d0*pi**2)*&
	 (4.d0*c23f(p1sq,p2sq,qsq,mf,mfp,mf)&
         +3.d0*c12f(p1sq,p2sq,qsq,mf,mfp,mf)&
         +c11f(p1sq,p2sq,qsq,mf,mfp,mf)&
         +c0f(p1sq,p2sq,qsq,mf,mfp,mf)&
	 )

	 end function
!--------------------------------------------
 	complex (kind(0d0)) function Gam3_hWW_fermion(p1sq,p2sq,qsq,mf)
 	double precision, intent(in):: p1sq,p2sq,qsq,mf
 	double precision:: mfp

       if (mf .eq. mt) mfp = mb
       if (mf .eq. mb) mfp = mt
       if (mf .eq. mc) mfp = ms
       if (mf .eq. ms) mfp = mc
       if (mf .eq. mtau) mfp = 0.d0

	Gam3_hWW_fermion =&
	 -4.d0*mw**4*mf**2/vev**3/(16.d0*pi**2)*&
	 (c11f(p1sq,p2sq,qsq,mf,mfp,mf)&
         +c12f(p1sq,p2sq,qsq,mf,mfp,mf)&
         +c0f(p1sq,p2sq,qsq,mf,mfp,mf)&
	 )

	 end function
!=================================================================
	complex (kind(0d0)) function Gam1_hZZ_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam1_hZZ_F_SM = nc*Gam1_hZZ_fermion(p1sq,p2sq,qsq,mt)&
                   +nc*Gam1_hZZ_fermion(p1sq,p2sq,qsq,mc)&
                   +nc*Gam1_hZZ_fermion(p1sq,p2sq,qsq,mb)&
                   +Gam1_hZZ_fermion(p1sq,p2sq,qsq,mtau)
	
	 end function

!--------------------------------------------
	complex (kind(0d0)) function Gam2_hZZ_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam2_hZZ_F_SM =  nc*Gam2_hZZ_fermion(p1sq,p2sq,qsq,mt)&
                   +nc*Gam2_hZZ_fermion(p1sq,p2sq,qsq,mc)&
                   +nc*Gam2_hZZ_fermion(p1sq,p2sq,qsq,mb)&
                   +Gam2_hZZ_fermion(p1sq,p2sq,qsq,mtau)
	
	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam3_hZZ_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam3_hZZ_F_SM = nc*Gam3_hZZ_fermion(p1sq,p2sq,qsq,mt)&
                   +nc*Gam3_hZZ_fermion(p1sq,p2sq,qsq,mc)&
                   +nc*Gam3_hZZ_fermion(p1sq,p2sq,qsq,mb)&
                   +Gam3_hZZ_fermion(p1sq,p2sq,qsq,mtau)

	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam1_hZZ_B_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	double precision:: p12
	p12=(qsq-p1sq-p2sq)/2.d0

	Gam1_hZZ_B_SM = &
	 -gz**2*1.d0/(16.d0*pi**2)*&
	 (&
	 +2.d0*lam_GpGmh_sm(mh)*(cw**2-sw**2)**2*c24f(p1sq,p2sq,qsq,mw,mw,mw)&
	 +2.d0*lam_GGh_sm(mh)*c24f(p1sq,p2sq,qsq,mz,mh,mz)&
	 +6.d0*lam_hhh_sm(mh)*c24f(p1sq,p2sq,qsq,mh,mz,mh)&
	 )&

	 +gz**2*1.d0/(16.d0*pi**2)*&
	 (&
	 +1.d0/2.d0*lam_GpGmh_sm(mh)*(cw**2-sw**2)**2*b0f(qsq,mw,mw)&
	 +1.d0/2.d0*lam_GGh_sm(mh)*b0f(qsq,mz,mz)&
	 +3.d0/2.d0*lam_hhh_sm(mh)*b0f(qsq,mh,mh)&
	 )&
!~~~~~~~~~~~ Gauge Boson Loops ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
	+1.d0/(16.d0*pi**2)*(&
	 +2.d0*g**3*mw*cw**2*ChVV1_VVV(p1sq,p2sq,qsq,mW,mW,mW)&
	 +g**3*mw*sw**2*ChVV1_SVV(p1sq,p2sq,qsq,mw,mW,mW)& 
	 +g**3*mw*sw**2*ChVV1_VVS(p1sq,p2sq,qsq,mW,mW,mw)&	 
	 -2.d0*g**3*mw**3/cw**2*sw**4*c0f(p1sq,p2sq,qsq,mW,mw,mW)&
	 -2.d0*g**3*cw**2*mw*c24f(p1sq,p2sq,qsq,mw,mw,mw)&
	 -g*gz**2*mw*(cw**2-sw**2)*sw**2*&
	 ( c24f(p1sq,p2sq,qsq,mW,mw,mw)+c24f(p1sq,p2sq,qsq,mw,mw,mW) )&
	 +2.d0*gz**2*lam_GpGmh_sm(mh)*mw**2*sw**4*c0f(p1sq,p2sq,qsq,mw,mW,mw)&
	 -6.d0*g**3*cw**2*mw*b0f(qsq,mW,mW)+4.d0*g**3*cw**2*mw&
	 -g*gz**2*mw*sw**4*( b0f(p2sq,mW,mw)+b0f(p1sq,mw,mW) )&
!
	 +gz**3/2.d0*mz*( c24f(p1sq,p2sq,qsq,mZ,mh,mz)+c24f(p1sq,p2sq,qsq,mz,mh,mZ) )&
	 -gz**3*mz**3*c0f(p1sq,p2sq,qsq,mZ,mh,mZ)&
	 +6.d0*lam_hhh_sm(mh)*gz**2*mz**2*c0f(p1sq,p2sq,qsq,mh,mZ,mh)&
	 -1.d0/2.d0*gz**3*mz*( b0f(p1sq,mh,mZ)+b0f(p2sq,mh,mZ))&
	 )

	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam2_hZZ_B_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	double precision:: p12
	p12=(qsq-p1sq-p2sq)/2.d0

	Gam2_hZZ_B_SM = gz**2/(16.d0*pi**2)*mz**2*(&
                   +2.d0*g*mw*cw**4*(ChVV2_VVV(p1sq,p2sq,qsq,mW,mW,mW) - C1223(p1sq,p2sq,qsq,mw,mw,mw)) &
!
                   +g*mw*sw**2*cw**2*ChVV2_SVV(p1sq,p2sq,qsq,mw,mW,mW)&
                   +g*mw*sw**2*cw**2*ChVV2_VVS(p1sq,p2sq,qsq,mW,mW,mw)&
!
                   -g*mw*(cw**2-sw**2)*sw**2*ChVV2_VSS(p1sq,p2sq,qsq,mW,mw,mw)&
                   -g*mw*(cw**2-sw**2)*sw**2*ChVV2_SSV(p1sq,p2sq,qsq,mw,mw,mW)&
                   +gz/2.d0*mz*ChVV2_VSS(p1sq,p2sq,qsq,mZ,mh,mz)&
                   +gz/2.d0*mz*ChVV2_SSV(p1sq,p2sq,qsq,mz,mh,mZ)&
! 
                   -2.d0*lam_GpGmh_sm(mh)*(cw**2-sw**2)**2*c1223(p1sq,p2sq,qsq,mw,mw,mw)&
                   -2.d0*lam_GGh_sm(mh)*c1223(p1sq,p2sq,qsq,mz,mh,mz)&
                   -6.d0*lam_hhh_sm(mh)*c1223(p1sq,p2sq,qsq,mh,mz,mh)&
                    )

	 end function
!==============================================================================
	complex (kind(0d0)) function Gam1_hWW_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam1_hWW_F_SM = nc*Gam1_hWW_fermion(p1sq,p2sq,qsq,mt)&
                   +nc*Gam1_hWW_fermion(p1sq,p2sq,qsq,mb)&
                   +nc*Gam1_hWW_fermion(p1sq,p2sq,qsq,mc)&
                   +nc*Gam1_hWW_fermion(p1sq,p2sq,qsq,ms)&
                   +Gam1_hWW_fermion(p1sq,p2sq,qsq,mtau)

	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam2_hWW_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam2_hWW_F_SM = nc*Gam2_hWW_fermion(p1sq,p2sq,qsq,mt)&
                   +nc*Gam2_hWW_fermion(p1sq,p2sq,qsq,mb)&
                   +nc*Gam2_hWW_fermion(p1sq,p2sq,qsq,mc)&
                   +nc*Gam2_hWW_fermion(p1sq,p2sq,qsq,ms)&
                   +Gam2_hWW_fermion(p1sq,p2sq,qsq,mtau)

	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam3_hWW_F_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam3_hWW_F_SM = nc*Gam3_hWW_fermion(p1sq,p2sq,qsq,mt)&
                   +nc*Gam3_hWW_fermion(p1sq,p2sq,qsq,mb)&
                   +nc*Gam3_hWW_fermion(p1sq,p2sq,qsq,mc)&
                   +nc*Gam3_hWW_fermion(p1sq,p2sq,qsq,ms)&
                      +Gam3_hWW_fermion(p1sq,p2sq,qsq,mtau)

	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam1_hWW_B_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	double precision:: p12
	double precision:: mgam,getlambda
	p12=(qsq-p1sq-p2sq)/2.d0

	Gam1_hWW_B_SM =&
	  g**2*1.d0/(16.d0*pi**2)*&
	 (&
	 -2.d0*lam_GGh_sm(mh)*c24f(p1sq,p2sq,qsq,mz,mw,mz)&
	 -6.d0*lam_hhh_sm(mh)*(c24f(p1sq,p2sq,qsq,mh,mw,mh) )&
	 -lam_GpGmh_sm(mh)*( c24f(p1sq,p2sq,qsq,mw,mz,mw)   )&
	 -lam_GpGmh_sm(mh)*( c24f(p1sq,p2sq,qsq,mw,mh,mw)   )&
	 )&
!
	 +g**2/(16.d0*pi**2)*&
	 (&
	 +1.d0/2.d0*lam_GpGmh_sm(mh)*b0f(qsq,mw,mw)&
	 +1.d0/2.d0*lam_GGh_sm(mh)*b0f(qsq,mz,mz)&
	 +3.d0/2.d0*lam_hhh_sm(mh)*b0f(qsq,mh,mh)&
	 )&
!~~~~~~~~~~~ Gauge Boson Loops ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~!
	  +1.d0/(16.d0*pi**2)*&
	 (&
	 +g**3*cw*mz*ChVV1_VVV(p1sq,p2sq,qsq,mZ,mW,mZ)&
	 +g**3*cw**2*mw*ChVV1_VVV(p1sq,p2sq,qsq,mW,mZ,mW)&
	 +g**3*sw**2*mw*ChVV1_VVV(p1sq,p2sq,qsq,mW,mphoton,mW)&
	 -1.d0/2.d0*g**3*mw*sw**2*(ChVV1_SVV(p1sq,p2sq,qsq,mw,mZ,mW)-ChVV1_SVV(p1sq,p2sq,qsq,mw,mphoton,mW))& 
	 -1.d0/2.d0*g**3*mw*sw**2*(ChVV1_VVS(p1sq,p2sq,qsq,mW,mZ,mw)-ChVV1_VVS(p1sq,p2sq,qsq,mW,mphoton,mw))& 
	 -g**3/cw**3*mw**2*mZ*sw**4*c0f(p1sq,p2sq,qsq,mZ,mw,mZ)&
	 -g**3*mw**3*c0f(p1sq,p2sq,qsq,mW,mh,mW)&
	 +6.d0*g**2*lam_hhh_sm(mh)*mw**2*c0f(p1sq,p2sq,qsq,mh,mW,mh)& 
	 +lam_GpGmh_sm(mh)*g**2*mw**2*sw**4/cw**2*c0f(p1sq,p2sq,qsq,mw,mZ,mw)&
	 +lam_GpGmh_sm(mh)*g**2*sw**2*mw**2*c0f(p1sq,p2sq,qsq,mw,mphoton,mw)&
	 +1.d0/2.d0*g**3*mw*( c24f(p1sq,p2sq,qsq,mW,mh,mw) + c24f(p1sq,p2sq,qsq,mw,mh,mW) )&
	 +1.d0/2.d0*g**3*sw**2/cw**2*mw*( c24f(p1sq,p2sq,qsq,mz,mw,mZ) + c24f(p1sq,p2sq,qsq,mZ,mw,mz) )&
!
	 -g**3*cw*mz*c24f(p1sq,p2sq,qsq,mz,mw,mz)&
	 -g**3*cw**2*mw*c24f(p1sq,p2sq,qsq,mw,mz,mw)&
	 -g**3*sw**2*mw*c24f(p1sq,p2sq,qsq,mw,mphoton,mw)&
	 -3.d0*g**3*mw*b0f(qsq,mW,mW)+2.d0*g**3*mw&
	 -3.d0*g**3*cw*mz*b0f(qsq,mZ,mZ)+2.d0*g**3*cw*mz&
!
	 -1.d0/2.d0*g**3*mw*( b0f(p1sq,mW,mh) + b0f(p2sq,mW,mh) )&
	 -1.d0/2.d0*g**3*mw*sw**4/cw**2*( b0f(p1sq,mZ,mw) + b0f(p2sq,mZ,mw) )&
	 -1.d0/2.d0*g**3*sw**2*mw*( b0f(p1sq,mphoton,mw) + b0f(p2sq,mphoton,mw) )&
	 )     


	 end function
!--------------------------------------------
	complex (kind(0d0)) function Gam2_hWW_B_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	double precision:: p12
	p12=(qsq-p1sq-p2sq)/2.d0

	Gam2_hWW_B_SM = g**2/(16.d0*pi**2)*mw**2*(&
                   +g*cw*mz*(ChVV2_VVV(p1sq,p2sq,qsq,mZ,mW,mZ) - c1223(p1sq,p2sq,qsq,mz,mw,mz)) &
                   +g*mw*cw**2*(ChVV2_VVV(p1sq,p2sq,qsq,mW,mZ,mW)-c1223(p1sq,p2sq,qsq,mw,mz,mw))& 
                   +g*mw*sw**2*(ChVV2_VVV(p1sq,p2sq,qsq,mW,mphoton,mW)-c1223(p1sq,p2sq,qsq,mw,mphoton,mw)) &
!
                    -g/2.d0*mw*(sw**2*ChVV2_SVV(p1sq,p2sq,qsq,mw,mZ,mW) - sw**2*ChVV2_SVV(p1sq,p2sq,qsq,mw,mphoton,mW))&
                    -g/2.d0*mw*(sw**2*ChVV2_VVS(p1sq,p2sq,qsq,mW,mZ,mw) - sw**2*ChVV2_VVS(p1sq,p2sq,qsq,mW,mphoton,mw))&
!
                    +g/2.d0*mw*ChVV2_VSS(p1sq,p2sq,qsq,mW,mh,mw)&
                    +g/2.d0*mw*ChVV2_SSV(p1sq,p2sq,qsq,mw,mh,mW)&
                    +g/2.d0*mw*sw**2/cw**2*ChVV2_VSS(p1sq,p2sq,qsq,mZ,mw,mz)&
                    +g/2.d0*mw*sw**2/cw**2*ChVV2_SSV(p1sq,p2sq,qsq,mz,mw,mZ)&
! 
                    -2.d0*lam_GGh_sm(mh)*c1223(p1sq,p2sq,qsq,mz,mw,mz)&
                    -1.d0*lam_GpGmh_sm(mh)*c1223(p1sq,p2sq,qsq,mw,mh,mw)&
                    -6.d0*lam_hhh_sm(mh)*c1223(p1sq,p2sq,qsq,mh,mw,mh)&
                    -1.d0*lam_GpGmh_sm(mh)*c1223(p1sq,p2sq,qsq,mw,mz,mw)&
                   )

	 end function
!##########################################################
!
!    SECTION III COUNTER-TERMS
!
!##########################################################
	double precision function del_mf(mf)
	double precision, intent(in):: mf
	double precision:: vf,af,qf,mfp
        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	del_mf = mf*(Piff_V_SM(mf**2,mf)+Piff_S_SM(mf**2,mf) )

	 end function
!---------------------------------------------
	double precision function del_ZVf(mf)
	double precision, intent(in):: mf
	double precision:: vf,af,qf,mfp
        if (mf .eq. mt) then 
        vf = vt
        af = at
        qf = qt
        mfp = mb
        elseif (mf .eq. mc) then 
        vf = vt
        af = at
        qf = qt
        mfp = ms
        elseif (mf .eq. mb) then 
        vf  = vb
        af  = ab
        qf  = qb
        mfp = mt
        elseif (mf .eq. mtau) then 
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        elseif (mf .eq. mmu) then ! add from v2
        vf = ve
        af = ae
        qf = qe
        mfp = 0.d0
        endif

	del_ZVf = -Piff_V_SM(mf**2,mf)-2.d0*mf**2*(dPiff_V_SM(mf**2,mf)+dPiff_S_SM(mf**2,mf))

	 end function
!---------------------------------------------
	double precision function del_swsq_SM()

	del_swsq_SM = cw**2*(PiZZ_F(mz**2)/mz**2-PiWW_F(mw**2)/mw**2)&
                     +cw**2*(PiZZ_B_SM(mz**2)/mz**2-PiWW_B_SM(mw**2)/mw**2) 

	 end function
!---------------------------------------------
	double precision function del_v_SM()

	del_v_SM = &
	 vev/2.d0*(&
	 PiWW_F(mw**2)/mw**2&
	 -delta_alpha &
         -(PiGG_F(mz**2)/mZ**2&
          +dPiGG_top(0.d0)-PiGG_top(mz**2)/mZ**2 )&
         )&
!
	 +vev/2.d0*(&
	 PiWW_B_SM(mw**2)/mw**2&
	 +del_swsq_SM()/sw**2&
	 -dPiGG_B_SM(0.d0)&
           )

	 end function
!---------------------------------------------
  	double precision function del_mZsq_SM()
	
	del_mZsq_SM = PiZZ_F(mz**2)+PiZZ_B_SM(mz**2)

	 end function
!--------------------------------------------
 	double precision function del_mWsq_SM()

	del_mWsq_SM = PiWW_F(mw**2) + PiWW_B_SM(mw**2)

	 end function
!--------------------------------------------
  	double precision function del_ZZ_SM()

	del_ZZ_SM =& 
	 -delta_alpha-(PiGG_F(mZ**2)/mZ**2&
          +dPiGG_top(0.d0)-PiGG_top(mZ**2)/mZ**2 )&
	   -2.d0*(cw**2-sw**2)/(cw*sw)*PiZG_F(0.d0)/mz**2&
            -(dPiGG_B_SM(0.d0)-dPiGG_pinch(0.d0))&
	   +2.d0*(cw**2-sw**2)/(cw*sw)*( PiZG_B_SM(0.d0)-PiZG_pinch(0.d0) )/mz**2&
	 +(cw**2-sw**2)/cw**2*del_swsq_SM()/sw**2

	 end function
!--------------------------------------------
  	double precision function del_ZW_SM()

	del_ZW_SM =&
	 -delta_alpha-(PiGG_F(mZ**2)/mZ**2&
          +dPiGG_top(0.d0)-PiGG_top(mZ**2)/mZ**2 )&
	   +2.d0*cw/sw*&
	 PiZG_F(0.d0)/mz**2&
	   -(dPiGG_B_SM(0.d0) -dPiGG_pinch(0.d0))&
	   +2.d0*cw/sw*(PiZG_B_SM(0.d0)-PiZG_pinch(0.d0))/mz**2&
	 +del_swsq_SM()/sw**2

	 end function
!--------------------------------------------
 	double precision function del_Th_SM()

 	del_Th_SM = -Th_F_SM()-Th_B_SM()

 	 end function
!---------------------------------------------
	double precision function del_mhsq_SM()
	
        del_mhsq_SM = del_Th_SM()/vev + Pihh_F_SM(mh**2)+Pihh_B_SM(mh**2)

        end function
!--------------------------------------------
	double precision function del_Zh_SM()

	del_Zh_SM = -dPihh_F_SM(mh**2) -dPihh_B_SM(mh**2)

	 end function
!--------------------------------------------
  	double precision function del_hhh_SM()

	del_hhh_SM = &
	 -6.d0*(&
	   (1.d0/vev)*del_mhsq_SM()/2.d0&
	 +3.d0/4.d0*(mh**2/vev)*del_Zh_SM()&
	 +mh**2/(2.d0*vev**3)*(-vev*del_v_SM())&
	)

	 end function
!--------------------------------------------
  	double precision function del_Gam1_hZZ_SM()

	del_Gam1_hZZ_SM = 2.d0*mz**2/vev*&
	 (&
	 +del_mzsq_SM()/mz**2&
	 +1.d0/2.d0*(del_Zh_SM()&
         +2.d0*del_ZZ_SM() )&
	 -del_v_SM()/vev&
	 )

	 end function
!--------------------------------------------
  	double precision function del_Gam1_hWW_SM()

	del_Gam1_hWW_SM = 2.d0*mw**2/vev*(&
	 +del_mWsq_SM()/mw**2&
	 +del_ZW_SM()&
	 +1.d0/2.d0*del_Zh_SM()&
	 -del_v_SM()/vev&
	 )

	 end function
!##########################################################
!
!    SECTION V RENORMALIZED HIGGS COUPLINGS     
!
!##########################################################
  	complex (kind(0d0)) function Gam_hhh_loop_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam_hhh_loop_SM =&
	 +Gam_hhh_F_SM(p1sq,p2sq,qsq)&
	 +Gam_hhh_B_SM(p1sq,p2sq,qsq)&
	 +del_hhh_SM()

	 end function
!---------------------------------------------
	complex (kind(0d0)) function Gam_hbb_loop_SM(p1sq,p2sq,qsq)

	double precision, intent(in):: p1sq,p2sq,qsq

	Gam_hbb_loop_SM =&
	+Gam_hff_S_SM(p1sq,p2sq,qsq,mb)&
	-mb/vev*(&
	del_mf(mb)/mb-del_v_SM()/vev&
	+del_ZVf(mb)+1.d0/2.d0*del_Zh_SM()& 
	)

	end function
!---------------------------------------------
	complex (kind(0d0)) function Gam_htt_loop_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam_htt_loop_SM =&
	+Gam_hff_S_SM(p1sq,p2sq,qsq,mt)&
	-mt/vev*(&
	del_mf(mt)/mt-del_v_SM()/vev&
	+del_ZVf(mt)+1.d0/2.d0*del_Zh_SM()& 
	)

	end function
!---------------------------------------------
	complex (kind(0d0)) function Gam_hcc_loop_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam_hcc_loop_SM =&
	+Gam_hff_S_SM(p1sq,p2sq,qsq,mc)&
	-mc/vev*(&
	del_mf(mc)/mc-del_v_SM()/vev&
	+del_ZVf(mc)+1.d0/2.d0*del_Zh_SM()& 
	)

	end function
!---------------------------------------------
	complex (kind(0d0)) function Gam_htautau_loop_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam_htautau_loop_SM =&
	+Gam_hff_S_SM(p1sq,p2sq,qsq,mtau)&
	-mtau/vev*(&
	del_mf(mtau)/mtau-del_v_SM()/vev&
	+del_ZVf(mtau)+1.d0/2.d0*del_Zh_SM()& 
	)

	end function
!---------------------------------------------
	complex (kind(0d0)) function Gam_hmumu_loop_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

	Gam_hmumu_loop_SM =&
	+Gam_hff_S_SM(p1sq,p2sq,qsq,mmu)&
	-mmu/vev*(&
	del_mf(mmu)/mmu-del_v_SM()/vev&
	+del_ZVf(mmu)+1.d0/2.d0*del_Zh_SM()& 
	)

	end function

!--------------------------------------------
  	complex (kind(0d0)) function Gam1_hZZ_loop_SM(p1sq,p2sq,qsq)
  	double precision, intent(in):: p1sq,p2sq,qsq
	
	Gam1_hZZ_loop_SM = &
	 +Gam1_hZZ_F_SM(p1sq,p2sq,qsq)&
	 +Gam1_hZZ_B_SM(p1sq,p2sq,qsq)&
     	 +del_Gam1_hZZ_SM()

	 end function
!--------------------------------------------
  	complex (kind(0d0)) function Gam1_hWW_loop_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	
	Gam1_hWW_loop_SM = &
	 +Gam1_hWW_F_SM(p1sq,p2sq,qsq)&
	 +Gam1_hWW_B_SM(p1sq,p2sq,qsq)&
	 +del_Gam1_hWW_SM()

	 end function
!==============================================================
  	complex (kind(0d0)) function rGam1_hZZ_SM(p1sq,p2sq,qsq)
  	double precision, intent(in):: p1sq,p2sq,qsq
	
	 rGam1_hZZ_SM = 2.d0*mZ**2/vev + Gam1_hZZ_loop_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
  	complex (kind(0d0)) function rGam2_hZZ_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

        rGam2_hZZ_SM =  Gam2_hZZ_F_SM(p1sq,p2sq,qsq) + Gam2_hZZ_B_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
   	complex (kind(0d0)) function rGam3_hZZ_SM(p1sq,p2sq,qsq)
 	double precision, intent(in):: p1sq,p2sq,qsq

         rGam3_hZZ_SM =  Gam3_hZZ_F_SM(p1sq,p2sq,qsq) 

         end function
!--------------------------------------------
  	complex (kind(0d0)) function rGam1_hWW_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq
	
	rGam1_hWW_SM = 2.d0*mW**2/vev + Gam1_hWW_loop_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
  	complex (kind(0d0)) function rGam2_hWW_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

        rGam2_hWW_SM =  Gam2_hWW_F_SM(p1sq,p2sq,qsq) + Gam2_hWW_B_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
   	complex (kind(0d0)) function rGam3_hWW_SM(p1sq,p2sq,qsq)
 	double precision, intent(in):: p1sq,p2sq,qsq

         rGam3_hWW_SM =  Gam3_hWW_F_SM(p1sq,p2sq,qsq) 

         end function
!--------------------------------------------
  	complex (kind(0d0)) function rGamS_hbb_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

        rGamS_hbb_SM = -mb/vev + Gam_hbb_loop_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
  	complex (kind(0d0)) function rGamS_htt_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

        rGamS_htt_SM = -mt/vev + Gam_htt_loop_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
  	complex (kind(0d0)) function rGamS_hcc_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

        rGamS_hcc_SM = -mc/vev + Gam_hcc_loop_SM(p1sq,p2sq,qsq)

	 end function
!--------------------------------------------
  	complex (kind(0d0)) function rGamS_htautau_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

         rGamS_htautau_SM = -mtau/vev + Gam_htautau_loop_SM(p1sq,p2sq,qsq)
 
	 end function
!--------------------------------------------
  	complex (kind(0d0)) function rGamS_hmumu_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

         rGamS_hmumu_SM = -mmu/vev + Gam_hmumu_loop_SM(p1sq,p2sq,qsq)
 
	 end function

!--------------------------------------------
  	complex (kind(0d0)) function rGam_hhh_SM(p1sq,p2sq,qsq)
	double precision, intent(in):: p1sq,p2sq,qsq

        rGam_hhh_SM = 6.d0*lam_hhh_SM(mh) + Gam_hhh_loop_SM(p1sq,p2sq,qsq)

	 end function
!##########################################################
!
!    SECTION VI decay rates for the Higgs boson
!
!##########################################################
!------------------------------------------------------
        double precision function decay_hbb_sm(mh,OREW,ORQCD)
        double precision, intent(in):: mh
        integer, intent(in):: OREW,ORQCD
        doubleprecision:: S_NLOEW,S_NLOQCD,S_NNLOQCD,S_RMSM
        double precision:: p1sq,p2sq,qsq,CF,mu,rmb

        p1sq=mb**2
        p2sq=mb**2
        qsq=mh**2
        CF = 4.d0/3.d0
        mu = mh
        rmb=mq_bar(mh,alphas(lam_QCD5,mh,5.d0),alphas(lam_QCD5,mb_MS,5.d0),mb_MS)

        call Switch_HOEW(OREW,S_NLOEW)
        call Switch_HOQCD(ORQCD,S_NLOQCD,S_NNLOQCD,S_RMSM)
 
        decay_hbb_sm=&
             3.d0*mh/8.d0/pi*(1.d0-4.d0*mb**2/mh**2)**(3.d0/2.d0)*&
             (mb/vev+S_RMSM*(rmb-mb)/vev)**2*(&
! LO
              1.d0&
! NLO Weak correction      
              +(-Del_r_SM()&
                +2.d0*(-vev/mb)*real(&
                Gam_hbb_loop_SM(p1sq,p2sq,qsq) &
                +mb*(Gam_hff_V1_SM(p1sq,p2sq,qsq,mb) -Gam_hff_V2_SM(p1sq,p2sq,qsq,mb) )&
                +mh**2*(1.d0-mb**2/mh**2)*Gam_hff_T_SM(p1sq,p2sq,qsq,mb) &
                ) )*S_NLOEW&
! NLO QED correction                                          
                 +alpha_em/pi*Qb**2*(17.d0/4.d0&
                       +3.d0/2.d0*log(mu**2/mh**2)+23.d0/2.d0*log(mu**2/mh**2)&
                      +9.d0/4.d0*log(mu**2/mh**2)**2 )*S_NLOEW&
! NLOQCD correction                                          
                  +alphas(lam_QCD5,mh,5.d0)/pi*CF*(17.d0/4.d0&
                        +3.d0/2.d0*log(mu**2/mh**2)+23.d0/2.d0*log(mu**2/mh**2)&
                       +9.d0/4.d0*log(mu**2/mh**2)**2 )*S_NLOQCD&
! NNLOQCD correction                                          
           +(alphas(lam_QCD5,mh,5.d0)/pi)**2*(35.94d0-1.36d0*5.d0)*S_NNLOQCD&
           +(alphas(lam_QCD5,mh,5.d0)/pi)**2*(1.57d0-2.d0/3.d0*log(mh**2/mt**2)&
            +1.d0/9.d0*(log(rmb**2/mh**2) )**2 )*S_NNLOQCD&
                )

        end function

!-------------------------------------------
        double precision function decay_hcc_sm(mh,OREW,ORQCD)
        double precision, intent(in):: mh
        integer, intent(in):: OREW,ORQCD
        doubleprecision:: S_NLOEW,S_NLOQCD,S_NNLOQCD,S_RMSM
        double precision:: p1sq,p2sq,qsq,CF,mu,rmc

        p1sq=mc**2
        p2sq=mc**2
        qsq=mh**2
        CF = 4.d0/3.d0
        mu = mh
        rmc= mq_bar(mh,alphas(lam_QCD5,mh,5.d0),alphas(lam_QCD4,mc_MS,4.d0),mc_MS)


        call Switch_HOEW(OREW,S_NLOEW)
        call Switch_HOQCD(ORQCD,S_NLOQCD,S_NNLOQCD,S_RMSM)

        decay_hcc_sm=&
              3.d0*mh/8.d0/pi*(1.d0-4.d0*mc**2/mh**2)**(3.d0/2.d0)*&
              (mc/vev+S_RMSM*(rmc-mc)/vev)**2*(&
! LO
             1.d0&
! NLO Weak correction
            +(-Del_r_SM()&
            +2.d0*(-vev/mc)*real(&
           +Gam_hcc_loop_SM(p1sq,p2sq,qsq)&
           +mc*(Gam_hff_V1_SM(p1sq,p2sq,qsq,mc)&
                  -Gam_hff_V2_SM(p1sq,p2sq,qsq,mc) )&
           +mh**2*(1.d0-mc**2/mh**2)*Gam_hff_T_SM(p1sq,p2sq,qsq,mc)&
               ) )*S_NLOEW&
! NLO QED correction                                          
                 +alpha_em/pi*Qt**2*(17.d0/4.d0&
                       +3.d0/2.d0*log(mu**2/mh**2)+23.d0/2.d0*log(mu**2/mh**2)&
                      +9.d0/4.d0*log(mu**2/mh**2)**2 )*S_NLOEW&
! NLO QCD correction                          
                +alphas(lam_QCD5,mh,5.d0)/pi*CF*(17.d0/4.d0&
                      +3.d0/2.d0*log(mu**2/mh**2)+23.d0/2.d0*log(mu**2/mh**2)&
                      +9.d0/4.d0*log(mu**2/mh**2)**2 )*S_NLOQCD&
! NNLO QCD correction                          
           +(alphas(lam_QCD5,mh,5.d0)/pi)**2*(35.94d0-1.36d0*5.d0)*S_NNLOQCD&
           +(alphas(lam_QCD5,mh,5.d0)/pi)**2*(1.57d0-2.d0/3.d0*log(mh**2/mt**2)&
            +1.d0/9.d0*(log(rmc**2/mh**2) )**2 )*S_NNLOQCD&
                )
             end function

!----------------------------------------------
        double precision function decay_htautau_sm(mh,OREW)
        double precision, intent(in):: mh
        integer, intent(in):: OREW
        doubleprecision:: S_NLOEW
        double precision:: p1sq,p2sq,qsq
        
        p1sq=mtau**2
        p2sq=mtau**2
        qsq=mh**2
        call Switch_HOEW(OREW,S_NLOEW)

        decay_htautau_sm=&
             mh/8.d0/pi*(1.d0-4.d0*mtau**2/mh**2)**(3.d0/2.d0)*&
             ( -mtau/vev)**2*(&
             1.d0&
! NLO Weak correction
             +(-Del_r_SM()&
             +2.d0*(-vev/mtau)*real(&
            +Gam_htautau_loop_SM(p1sq,p2sq,qsq)&
            +mtau*(Gam_hff_V1_SM(p1sq,p2sq,qsq,mtau)&
                   -Gam_hff_V2_SM(p1sq,p2sq,qsq,mtau) )&
            +mh**2*(1.d0-mtau**2/mh**2)*Gam_hff_T_SM(p1sq,p2sq,qsq,mtau)&
                ) )*S_NLOEW&
! NLO QED correction
                +Del_QED_hff(mtau)*S_NLOEW&  ! NLO QED correction (exaxt formula)
                )

             end function
!----------------------------------------------
        double precision function decay_hmumu_sm(mh,OREW)
        double precision, intent(in):: mh
        integer, intent(in):: OREW
        doubleprecision:: S_NLOEW
        double precision:: p1sq,p2sq,qsq
        
        p1sq=mmu**2
        p2sq=mmu**2
        qsq=mh**2
        call Switch_HOEW(OREW,S_NLOEW)

        decay_hmumu_sm=&
             mh/8.d0/pi*(1.d0-4.d0*mmu**2/mh**2)**(3.d0/2.d0)*&
             ( -mmu/vev)**2*(&
             1.d0&
! NLO Weak correction
             +(-Del_r_SM()&
             +2.d0*(-vev/mmu)*real(&
            +Gam_hmumu_loop_SM(p1sq,p2sq,qsq)&
            +mmu*(Gam_hff_V1_SM(p1sq,p2sq,qsq,mmu)&
                   -Gam_hff_V2_SM(p1sq,p2sq,qsq,mmu) )&
            +mh**2*(1.d0-mmu**2/mh**2)*Gam_hff_T_SM(p1sq,p2sq,qsq,mmu)&
                ) )*S_NLOEW&
! NLO QED correction
                +Del_QED_hff(mmu)*S_NLOEW&  ! NLO QED correction (exaxt formula)
                )

             end function
!----------------------------------------------- 
        double precision function decay_hgamgam_sm(mh,ORQCD)
        double precision, intent(in):: mh       
        integer, intent(in):: ORQCD
        doubleprecision:: S_NLOQCD,S_NNLOQCD,S_RMSM
        call Switch_HOQCD(ORQCD,S_NLOQCD,S_NNLOQCD,S_RMSM)

        decay_hgamgam_sm=alpha_em**2*G_F/(128.d0*dsqrt(2.d0)*pi**3)*mh**3*&
        ABS(&
        Nc*Qt**2*I_f(mh,mt)*(&
        +1.d0&
        -1.d0*alphas(lam_QCD5,mh/2.d0,5.d0)/pi*S_NLOQCD& !NLO QCD
        -(alphas(lam_QCD5,mh/2.d0,5.d0)/pi)**2&
         *(31.d0/24.d0+7.d0/4.d0*log((mh/2.d0)**2/mt**2) )*S_NNLOQCD)&    !NNLO QCD
        +Nc*Qb**2*I_f(mh,mb)+Qe**2*I_f(mh,mtau)+I_W(mh)&
        )**2
         end function
!----------------------------------------
        double precision function decay_hZgam_sm(mh,ORQCD)
        double precision, intent(in):: mh
        integer, intent(in):: ORQCD
        doubleprecision:: S_NLOQCD,S_NNLOQCD,S_RMSM
        call Switch_HOQCD(ORQCD,S_NLOQCD,S_NNLOQCD,S_RMSM)

        decay_hZgam_sm=dsqrt(2.d0)*G_F*alpha_em**2*mh**3/(128.d0*pi**3)*&
        (1.d0-mz**2/mh**2)**3*&
        abs(&
         Nc*Qt*vt*J_f(mh,mt)*(&
         1.d0&
         -1.d0*alphas(lam_QCD5,mh/2.d0,5.d0)/pi*S_NLOQCD& !NLO QCD
         +0.d0*S_NNLOQCD)&                                !NNLO QCD
        +Nc*Qb*vb*J_f(mh,mb)+Qe*ve*J_f(mh,mtau)+J_W(mh)&
        )**2
         end function
!---------------------------------------- 
        double precision function decay_hgg_sm(mh,ORQCD)
        double precision, intent(in):: mh
        integer, intent(in):: ORQCD
        doubleprecision:: S_NLOQCD,S_NNLOQCD,S_RMSM,AS
        call Switch_HOQCD(ORQCD,S_NLOQCD,S_NNLOQCD,S_RMSM)
        AS = alpha_s+(alphas(lam_QCD5,mh,5.d0)-alpha_s)*S_RMSM

        decay_hgg_sm=sqrt(2.d0)*G_F*AS**2*mh**3/(128.d0*pi**3)*&
        ABS(I_f(mh,mt)+I_f(mh,mb))**2&
        *(1.d0&
          +alphas(lam_QCD5,mh,5.d0)/pi*(95.d0/4.d0-7.d0/6.d0*5.d0& !NLO QCD
          +(33.d0-2.d0*5.d0)/6.d0*log(mh**2/mh**2) )*S_NLOQCD& 
          +(156.808d0-5.708d0*log(mt**2/mh**2))*(alphas(lam_QCD5,mh,5.d0)/pi)**2*S_NNLOQCD ) !NNLO QCD

         end function
!------------------------------------------------------------------------------
        double precision function decay_hWWstar_tree(mh)
        double precision, intent(in):: mh
        double precision:: epsi
        epsi=mW/mh

        decay_hWWstar_tree=&
          sqrt(2.d0)*G_F*mh*(2.d0*mW**2/vev)**2/768.d0/pi**3*(&
          3.d0*(1.d0-8.d0*epsi**2+20.d0*epsi**4)/sqrt(4.d0*epsi**2-1.d0)*&
          acos((3.d0*epsi**2-1.d0)/2.d0/epsi**3)  &
          -(1.d0-epsi**2)*(47.d0/2.d0*epsi**2-13.d0/2.d0+1.d0/epsi**2)&
          -3.d0*(1.d0-6.d0*epsi**2+4.d0*epsi**4)*Log(epsi)&
          )*18.d0
        end function

!------------------------------------------------------------------------------
        double precision function decay_hZZstar_tree(mh)
        double precision, intent(in):: mh
        double precision:: epsi
        epsi=mZ/mh

        decay_hZZstar_tree=&
         4.d0*sqrt(2.d0)*G_F*mh*(2.d0*mZ**2/vev)**2/768.d0/pi**3*(&
          3.d0*(1.d0-8.d0*epsi**2+20.d0*epsi**4)/sqrt(4.d0*epsi**2-1.d0)*&
          acos((3.d0*epsi**2-1.d0)/2.d0/epsi**3)  &
          -(1.d0-epsi**2)*(47.d0/2.d0*epsi**2-13.d0/2.d0+1.d0/epsi**2)&
          -3.d0*(1.d0-6.d0*epsi**2+4.d0*epsi**4)*Log(epsi)&
          )&
        *9.d0/2.d0*(7.d0/12.d0-10.d0/9.d0*sW**2+40.d0/27.d0*sW**4) 
        end function

!------------------------------------------------------------------------------
        double precision function decay_hZee_tree(mh)
        double precision, intent(in):: mh
        double precision:: epsi
        epsi=mZ/mh

        decay_hZee_tree=&
          4.d0*(ve**2+ae**2)*sqrt(2.d0)*G_F*mh*(2.d0*mZ**2/vev)**2/768.d0/pi**3*(&
          3.d0*(1.d0-8.d0*epsi**2+20.d0*epsi**4)/sqrt(4.d0*epsi**2-1.d0)*&
          acos((3.d0*epsi**2-1.d0)/2.d0/epsi**3)  &
          -(1.d0-epsi**2)*(47.d0/2.d0*epsi**2-13.d0/2.d0+1.d0/epsi**2)&
          -3.d0*(1.d0-6.d0*epsi**2+4.d0*epsi**4)*Log(epsi)&
          )

        end function
!------------------------------------------------------------------------------
        double precision function decay_hZnunu_tree(mh)
        double precision, intent(in):: mh
        double precision:: epsi
        epsi=mZ/mh

        decay_hZnunu_tree=&
         4.d0*(vnu**2+anu**2)*sqrt(2.d0)*G_F*mh*(2.d0*mZ**2/vev)**2/768.d0/pi**3*(&
          3.d0*(1.d0-8.d0*epsi**2+20.d0*epsi**4)/sqrt(4.d0*epsi**2-1.d0)*&
          acos((3.d0*epsi**2-1.d0)/2.d0/epsi**3)  &
          -(1.d0-epsi**2)*(47.d0/2.d0*epsi**2-13.d0/2.d0+1.d0/epsi**2)&
          -3.d0*(1.d0-6.d0*epsi**2+4.d0*epsi**4)*Log(epsi)&
          )
        end function


!-------- Zff and Wff vertex function at the 1-loop level --------------
       complex (kind(0d0)) function Fa_Vff(p1sq,p2sq,qsq,mX,mY,mZ)
       double precision, intent(in):: qsq,p1sq,p2sq,mX,mY,mZ

       Fa_Vff= &
           2.d0*qsq*(c11f(p1sq,p2sq,qsq,mX,mY,mZ)+c23f(p1sq,p2sq,qsq,mX,mY,mZ))&
          +4.d0*c24f(p1sq,p2sq,qsq,mX,mY,mZ)-2.d0!&
          ! Bellow is terms which are appeared when we evaluate mf\neq0 
          ! These terms is not nesecery since h->Vff is calcurated, neglecting terms proportional to mf^2. 
!             +2.d0*mX**2*(c21f(p1sq,p2sq,qsq,mX,mY,mZ)&
!                         +c22f(p1sq,p2sq,qsq,mX,mY,mZ)&
!                         -2.d0*c23f(p1sq,p2sq,qsq,mX,mY,mZ)&
!                         -2.d0*c11f(p1sq,p2sq,qsq,mX,mY,mZ)&
!                         +2.d0*c12f(p1sq,p2sq,qsq,mX,mY,mZ)&
!                         -c0f(p1sq,p2sq,qsq,mX,mY,mZ) ) 
       end function
!------------------
       complex (kind(0d0)) function Fa_Wffp_photon(p1sq,p2sq,qsq,mX,mY,mZ)
       double precision, intent(in):: qsq,p1sq,p2sq,mX,mY,mZ

       Fa_Wffp_photon= &
           2.d0*qsq*(c11f(p1sq,p2sq,qsq,mX,mY,mZ)+c23f(p1sq,p2sq,qsq,mX,mY,mZ))&
          +4.d0*c24f(p1sq,p2sq,qsq,mX,mY,mZ)-2.d0&
          ! Bellow is terms which are appeared when we evaluate mf\neq0 
            +2.d0*mX**2*(c21f(p1sq,p2sq,qsq,mX,mY,mZ)&
                         -c23f(p1sq,p2sq,qsq,mX,mY,mZ) )&
            +2.d0*mZ**2*(c22f(p1sq,p2sq,qsq,mX,mY,mZ)&
                        -c23f(p1sq,p2sq,qsq,mX,mY,mZ)&
                        -c11f(p1sq,p2sq,qsq,mX,mY,mZ)&
                        +c12f(p1sq,p2sq,qsq,mX,mY,mZ) )
       end function

!----------------------------------------------------------------
       complex (kind(0d0)) function Fb_Vff(p1sq,p2sq,qsq,mX,mY,mZ)
       double precision, intent(in):: qsq,p1sq,p2sq,mX,mY,mZ

       Fb_Vff= &
           2.d0*( qsq*(c11f(p1sq,p2sq,qsq,mX,mY,mZ)+c23f(p1sq,p2sq,qsq,mX,mY,mZ)&
            +c0f(p1sq,p2sq,qsq,mX,mY,mZ) )&
          +(6.d0)*c24f(p1sq,p2sq,qsq,mX,mY,mZ)-1.d0&
             )
       end function
!----------------------------------------------------------------
       complex (kind(0d0)) function Fb1_Wffp_phpton(p1sq,p2sq,qsq,mX,mY,mZ)
       double precision, intent(in):: qsq,p1sq,p2sq,mX,mY,mZ

       Fb1_Wffp_phpton= &
           2.d0*( qsq*(c11f(p1sq,p2sq,qsq,mX,mY,mZ)+c23f(p1sq,p2sq,qsq,mX,mY,mZ)&
            +c0f(p1sq,p2sq,qsq,mX,mY,mZ) )&
          +(6.d0)*c24f(p1sq,p2sq,qsq,mX,mY,mZ)-1.d0&
          +p1sq*(c21f(p1sq,p2sq,qsq,mX,mY,mZ)-c23f(p1sq,p2sq,qsq,mX,mY,mZ)-1.d0/2.d0*c0f(p1sq,p2sq,qsq,mX,mY,mZ)&
           +1.d0/2.d0*c11f(p1sq,p2sq,qsq,mX,mY,mZ) )&
          +p2sq*(c22f(p1sq,p2sq,qsq,mX,mY,mZ)-c23f(p1sq,p2sq,qsq,mX,mY,mZ)-c11f(p1sq,p2sq,qsq,mX,mY,mZ)&
          +1.d0/2.d0*c12f(p1sq,p2sq,qsq,mX,mY,mZ)+1.d0*2.d0*c0f(p1sq,p2sq,qsq,mX,mY,mZ)  )&
             )
       end function

!----------------------------------------------------------------
       complex (kind(0d0)) function Fb2_Wffp_phpton(p1sq,p2sq,qsq,mX,mY,mZ)
       double precision, intent(in):: qsq,p1sq,p2sq,mX,mY,mZ

       Fb2_Wffp_phpton= &
           2.d0*( qsq*(c11f(p1sq,p2sq,qsq,mX,mY,mZ)+c23f(p1sq,p2sq,qsq,mX,mY,mZ)&
            +c0f(p1sq,p2sq,qsq,mX,mY,mZ) )&
          +(6.d0)*c24f(p1sq,p2sq,qsq,mX,mY,mZ)-1.d0&
          +p1sq*(c21f(p1sq,p2sq,qsq,mX,mY,mZ)-c23f(p1sq,p2sq,qsq,mX,mY,mZ)+c0f(p1sq,p2sq,qsq,mX,mY,mZ)&
          +1.d0/2.d0*c11f(p1sq,p2sq,qsq,mX,mY,mZ))&
          +p2sq*(c22f(p1sq,p2sq,qsq,mX,mY,mZ)-c23f(p1sq,p2sq,qsq,mX,mY,mZ)-c11f(p1sq,p2sq,qsq,mX,mY,mZ)&
          +1.d0/2.d0*c12f(p1sq,p2sq,qsq,mX,mY,mZ)-c0f(p1sq,p2sq,qsq,mX,mY,mZ)  )&
             )
       end function
!------------------
       complex (kind(0d0)) function Fbf_photon(qsq,mX,mY)
       double precision, intent(in):: qsq,mX,mY

       Fbf_photon= &
            2.d0*b1f(qsq,mX,mY)+1.d0+4.d0*mX**2*(db1f(qsq,mX,mY)+2.d0*db0f(qsq,mX,mY))
       end function

!----------------------------------------------------------------
       complex (kind(0d0)) function rGamV_Zff_loop_SM(qsq,vf,af,Iff)
       double precision, intent(in):: vf,af,Iff,qsq

       rGamV_Zff_loop_SM= &
            1.d0/16.d0/pi**2*(&

             -gz**3*vf*(vf**2+3.d0*af**2)*(Fa_Vff(0.d0,0.d0,qsq,0.d0,mZ,0.d0)&
             +2.d0*b1f(0.d0,0.d0,mZ)+1.d0)&
             -g**2/4.d0*gz*(vf+af)*(Fa_Vff(0.d0,0.d0,qsq,0.d0,mW,0.d0)&
             +2.d0*b1f(0.d0,0.d0,mW)+1.d0)&
             +Iff/2.d0*g**2*gz*cw**2*(Fa_Vff(0.d0,0.d0,qsq,0.d0,mW,0.d0)  &
             -Fb_Vff(0.d0,0.d0,qsq,mW,0.d0,mW)+2.d0*b0f(0.d0,mW,mW))&
       )
       end function
!----------------------------------------------------------------
       complex (kind(0d0)) function rGamA_Zff_loop_SM(qsq,vf,af,Iff)
       double precision, intent(in):: vf,af,Iff,qsq

       rGamA_Zff_loop_SM= &
            1.d0/16.d0/pi**2*(&
            +gz**3*af*(3.d0*vf**2+af**2)*(Fa_Vff(0.d0,0.d0,qsq,0.d0,mZ,0.d0)&
            +2.d0*b1f(0.d0,0.d0,mZ)+1.d0)&
            +g**2/4.d0*gz*(vf+af)*(Fa_Vff(0.d0,0.d0,qsq,0.d0,mW,0.d0)&
            +2.d0*b1f(0.d0,0.d0,mW)+1.d0)&
            -Iff/2.d0*g**2*gz*cw**2*(Fa_Vff(0.d0,0.d0,qsq,0.d0,mW,0.d0)&
            -Fb_Vff(0.d0,0.d0,qsq,mW,0.d0,mW)+2.d0*b0f(0.d0,mW,mW))&
       )
       end function
!----------------------------------------------------------------
       complex (kind(0d0)) function rGam_Wffp_loop_SM(qsq,mf)
       double precision, intent(in):: qsq,mf
       double precision:: vf,af,Qf,vfp,afp,Qfp,mfp
       double precision:: mgam

       mgam = mphoton
        if (mf .eq. mc) then 
        vf = vt ; af = at ; Qf = Qt
        vfp = vb ; afp = ab ; Qfp = Qb ; mfp = msir
        elseif (mf .eq. msir) then 
        vf = vb ; af = ab ; Qf = Qb
        vfp  = vt ; afp  = at ; Qfp  = Qt ; mfp = mc
        elseif (mf .eq. mtau) then 
        vf = ve ; af = ae ; Qf = Qe
        vfp = vnu ; afp = anu ; Qfp = 0.d0 ; mfp = 0.d0
        elseif (mf .eq. mmu) then 
        vf = ve ; af = ae ; Qf = Qe 
        vfp = vnu ; afp = anu ; Qfp = 0.d0 ; mfp = 0.d0
        endif

       rGam_Wffp_loop_SM= &
            -1.d0/16.d0/pi**2*g/sqrt(2.d0)*(&
             Qf*Qfp*(g*sw)**2*(&
                          Fa_Wffp_photon(mf**2,mfp**2,qsq,mf,mgam,mfp)&     
                         +1.d0/2.d0*(Fbf_photon(mf**2,mf,mgam)+ Fbf_photon(mfp**2,mfp,mgam) )&     
                          )& 
            +gZ**2*(vf+af)*(vfp+afp)*(Fa_Vff(mf**2,mfp**2,qsq,mf,mZ,mfp)&
                         +b1f(mf**2,mf,mZ)+1.d0/2.d0+b1f(mfp**2,mfp,mZ)+1.d0/2.d0 )&  
              +4.d0*(g*sw)**2*af*Qf*(Fb2_Wffp_phpton(mf**2,mfp**2,qsq,mgam,mf,mW)&                
                               -Fb_Vff(mf**2,mfp**2,qsq,mZ,mf,mW) &
                               )&
              +4.d0*(g*sw)**2*afp*Qfp*(Fb1_Wffp_phpton(mf**2,mfp**2,qsq,mW,mfp,mgam)&             
                               -Fb_Vff(mf**2,mfp**2,qsq,mW,mfp,mZ) &
                               )&
            +1.d0/2.d0*(g*sw)**2*(&
                         +1.d0/2.d0*(Fbf_photon(mf**2,mf,mgam)+Fbf_photon(mfp**2,mfp,mgam) )&    
                            -b1f(mf**2,mf,mZ)-1.d0/2.d0 -b1f(mfp**2,mfp,mZ)-1.d0/2.d0 &
                               )&
            +1.d0/2.d0*g**2*(Fb_Vff(mf**2,mfp**2,qsq,mZ,mf,mW)&
                            +Fb_Vff(mf**2,mfp**2,qsq,mW,mfp,mZ)&
                            -4.d0*b0f(0.d0,mW,mW)&
                            +b1f(mf**2,mf,mW)+1.d0/2.d0+b1f(mfp**2,mfp,mW)+1.d0/2.d0&
                            +b1f(mf**2,mf,mZ)+1.d0/2.d0+b1f(mfp**2,mfp,mZ)+1.d0/2.d0 )&
           +1.d0/4.d0*(Qf**2-Qfp**2)*(g*sw)**2*(Fbf_photon(mf**2,mf,mgam)-Fbf_photon(mfp**2,mfp,mgam) )&   
           +1.d0/2.d0*((vf+af)**2-(vfp+afp)**2)*gZ**2*(b1f(mf**2,mf,mZ)-b1f(mfp**2,mfp,mZ) )&
       ) 


        end function

!--- function at the 1-loop level --------------
       complex (kind(0d0)) function rGam1_hZgam_fermion(p1sq,p2sq,qsq,mf)
       double precision, intent(in):: qsq,p1sq,p2sq,mf
       double precision:: vf,af,qf
       if (mf .eq. mt) then 
        vf = vt ; af = at ; qf=qt
       elseif (mf .eq. mc) then 
        vf = vt ; af = at ; qf=qt
       elseif (mf .eq. mb) then 
        vf = vb ; af = ab ; qf=qb
       elseif (mf .eq. mtau) then 
        vf = ve ; af = ae ; qf=qe
       elseif (mf .eq. mmu) then 
        vf = ve ; af = ae ; qf=qe
       elseif (mf .eq. ms) then 
        vf = vb ; af = ab ; qf=qb
       endif


       rGam1_hZgam_fermion=&
            (g*sw)*g*gz*mW*1.d0/16.d0/pi**2*4.d0*Qf*vf*(mf**2/mW**2)*(&
            -1.d0/2.d0*(p1sq+p2sq-qsq)*c0f(p1sq,p2sq,qsq,mf,mf,mf)&
            -4.d0*(c24f(p1sq,p2sq,qsq,mf,mf,mf)-1.d0/4.d0*b0f(qsq,mf,mf)) &
            )
       end function
!----------------------------------------------------------------
       complex (kind(0d0)) function rGam2_hZgam_fermion(p1sq,p2sq,qsq,mf)
       double precision, intent(in):: qsq,p1sq,p2sq,mf
       double precision:: vf,af,qf
       if (mf .eq. mt) then 
        vf = vt ; af = at ; qf=qt
       elseif (mf .eq. mc) then 
        vf = vt ; af = at ; qf=qt
       elseif (mf .eq. mb) then 
        vf = vb ; af = ab ; qf=qb
       elseif (mf .eq. mtau) then 
        vf = ve ; af = ae ; qf=qe
       elseif (mf .eq. mmu) then 
        vf = ve ; af = ae ; qf=qe
       elseif (mf .eq. ms) then 
        vf = vb ; af = ab ; qf=qb
       endif


       rGam2_hZgam_fermion=&
            (g*sw)*g*gz*mW*1.d0/16.d0/pi**2*4.d0*Qf*vf*(mf**2/mW**2)*(&
            -c0f(p1sq,p2sq,qsq,mf,mf,mf)&
            -4.d0*(c23f(p1sq,p2sq,qsq,mf,mf,mf)+c12f(p1sq,p2sq,qsq,mf,mf,mf) ) &
            )
       end function

!------------------------------------------------------------------
       complex (kind(0d0)) function rGam1_hZgam_F_SM(p1sq,p2sq,qsq)
       double precision, intent(in):: qsq,p1sq,p2sq

       rGam1_hZgam_F_SM=&
            nc*rGam1_hZgam_fermion(p1sq,p2sq,qsq,mt)&
           +nc*rGam1_hZgam_fermion(p1sq,p2sq,qsq,mb)&
           +nc*rGam1_hZgam_fermion(p1sq,p2sq,qsq,mc)&
            +rGam1_hZgam_fermion(p1sq,p2sq,qsq,mtau)!&
!            +rGam1_hZgam_fermion(p1sq,p2sq,qsq,mmu)&  ! contributions from light fermions are omitted
!            +nc*rGam1_hZgam_fermion(p1sq,p2sq,qsq,ms)
       end function
!-----------------------------------------------------------------
       complex (kind(0d0)) function rGam2_hZgam_F_SM(p1sq,p2sq,qsq)
       double precision, intent(in):: qsq,p1sq,p2sq

       rGam2_hZgam_F_SM=&
            nc*rGam2_hZgam_fermion(p1sq,p2sq,qsq,mt)&
           +nc*rGam2_hZgam_fermion(p1sq,p2sq,qsq,mb)&
           +nc*rGam2_hZgam_fermion(p1sq,p2sq,qsq,mc)&
            +rGam2_hZgam_fermion(p1sq,p2sq,qsq,mtau)!&
!            +rGam2_hZgam_fermion(p1sq,p2sq,qsq,mmu)&
!            +nc*rGam2_hZgam_fermion(p1sq,p2sq,qsq,ms)
       end function
!------------------------------------------------------------------
       complex (kind(0d0)) function rGam1_hZgam_W_SM(p1sq,p2sq,qsq)
       double precision, intent(in):: qsq,p1sq,p2sq

       rGam1_hZgam_W_SM=&
            (g*sw)*g*gz*mW*1.d0/16.d0/pi**2*(&
            2.d0*(b0f(p2sq,mW,mW)-b0f(0.d0,mW,mW))&
            +(cw**2*(5.d0*p1sq+5.d0*p2sq-7.d0*qsq)-2.d0*p1sq+p2sq+qsq&
            +3.d0*mW**2+mh**2*sw**2)*c0f(p1sq,p2sq,qsq,mW,mW,mW)&
            +2.d0*(12.d0*cw**2-2.d0+(2.d0*cw**2-1.d0)*mh**2/mW**2)*&
            (c24f(p1sq,p2sq,qsq,mW,mW,mW)-1.d0/4.d0*b0f(qsq,mW,mW))&
            )
       end function
!-----------------------------------------------------------------
       complex (kind(0d0)) function rGam2_hZgam_W_SM(p1sq,p2sq,qsq)
       double precision, intent(in):: qsq,p1sq,p2sq

       rGam2_hZgam_W_SM=&
           (g*sw)*g*gz*mW*1.d0/16.d0/pi**2*(&
           4.d0*(4.d0*cw**2-1.d0)*c0f(p1sq,p2sq,qsq,mW,mW,mW)&
           +2.d0*(12.d0*cw**2-2.d0+mh**2/mW**2*(2.d0*cw**2-1.d0))*&
          (c23f(p1sq,p2sq,qsq,mW,mW,mW)+c12f(p1sq,p2sq,qsq,mW,mW,mW) )&
           )
       end function

!-----------------------------------------------------------------
       complex (kind(0d0)) function rGam2_hZgam_SM(p1sq,p2sq,qsq)
       double precision, intent(in):: qsq,p1sq,p2sq

       rGam2_hZgam_SM=&
            rGam2_hZgam_F_SM(p1sq,p2sq,qsq)&
           +rGam2_hZgam_W_SM(p1sq,p2sq,qsq)

       end function
!-----------------------------------------------------------------
       complex (kind(0d0)) function rGam1_hZgam_SM(p1sq,p2sq,qsq)
       double precision, intent(in):: qsq,p1sq,p2sq

       rGam1_hZgam_SM=&
            rGam1_hZgam_F_SM(p1sq,p2sq,qsq)&
           +rGam1_hZgam_W_SM(p1sq,p2sq,qsq)

       end function
!-----------------------------------------------------------------
       complex (kind(0d0)) function rGam1_hgamgam_fermion(p1sq,p2sq,qsq,mf)
       double precision, intent(in):: qsq,p1sq,p2sq,mf
       double precision:: qf
       if (mf .eq. mt) then 
        qf=qt
       elseif (mf .eq. mc) then 
        qf=qt
       elseif (mf .eq. mb) then 
        qf=qb
       elseif (mf .eq. mtau) then 
        qf=qe
       endif


       rGam1_hgamgam_fermion=&
            (g*sw)**2*g*mW*1.d0/16.d0/pi**2*4.d0*Qf**2*(mf**2/mW**2)*(&
            -1.d0/2.d0*(p1sq+p2sq-qsq)*c0f(p1sq,p2sq,qsq,mf,mf,mf)&
            -4.d0*(c24f(p1sq,p2sq,qsq,mf,mf,mf)-1.d0/4.d0*b0f(qsq,mf,mf)) &
            )
       end function
!----------------------------------------------------------------
       complex (kind(0d0)) function rGam2_hgamgam_fermion(p1sq,p2sq,qsq,mf)
       double precision, intent(in):: qsq,p1sq,p2sq,mf
       double precision:: vf,af,qf
       if (mf .eq. mt) then 
        qf=qt
       elseif (mf .eq. mc) then 
        qf=qt
       elseif (mf .eq. mb) then 
        qf=qb
       elseif (mf .eq. mtau) then 
        qf=qe
       endif


       rGam2_hgamgam_fermion=&
            (g*sw)**2*g*mW*1.d0/16.d0/pi**2*4.d0*Qf**2*(mf**2/mW**2)*(&
            -c0f(p1sq,p2sq,qsq,mf,mf,mf)&
            -4.d0*(c23f(p1sq,p2sq,qsq,mf,mf,mf)+c12f(p1sq,p2sq,qsq,mf,mf,mf) ) &
            )
       end function

!------------------------------------------------------------------
       complex (kind(0d0)) function rGam1_hgamgam_F_SM(p1sq,p2sq,qsq)
       double precision, intent(in):: qsq,p1sq,p2sq

       rGam1_hgamgam_F_SM=&
            nc*rGam1_hgamgam_fermion(p1sq,p2sq,qsq,mt)&
           +nc*rGam1_hgamgam_fermion(p1sq,p2sq,qsq,mb)&
           +nc*rGam1_hgamgam_fermion(p1sq,p2sq,qsq,mc)&
            +rGam1_hgamgam_fermion(p1sq,p2sq,qsq,mtau)
       end function
!-----------------------------------------------------------------
       complex (kind(0d0)) function rGam2_hgamgam_F_SM(p1sq,p2sq,qsq)
       double precision, intent(in):: qsq,p1sq,p2sq

       rGam2_hgamgam_F_SM=&
            nc*rGam2_hgamgam_fermion(p1sq,p2sq,qsq,mt)&
           +nc*rGam2_hgamgam_fermion(p1sq,p2sq,qsq,mb)&
           +nc*rGam2_hgamgam_fermion(p1sq,p2sq,qsq,mc)&
            +rGam2_hgamgam_fermion(p1sq,p2sq,qsq,mtau)
       end function

!------------------------------------------------------------------
       complex (kind(0d0)) function rGam1_hgamgam_W_SM(p1sq,p2sq,qsq)
       double precision, intent(in):: qsq,p1sq,p2sq

       rGam1_hgamgam_W_SM=&
            -(g*sw)**2*g*mW*1.d0/16.d0/pi**2*(&
            +(-5.d0*p1sq-5.d0*p2sq+7.d0*qsq+mh**2)*c0f(p1sq,p2sq,qsq,mW,mW,mW)&
            -4.d0*(6.d0+mh**2/mW**2)&
            *(c24f(p1sq,p2sq,qsq,mW,mW,mW)-1.d0/4.d0*b0f(qsq,mW,mW))&
            )
       end function
!-----------------------------------------------------------------
       complex (kind(0d0)) function rGam2_hgamgam_W_SM(p1sq,p2sq,qsq)
       double precision, intent(in):: qsq,p1sq,p2sq

       rGam2_hgamgam_W_SM=&
           4.d0*(g*sw)**2*g*mW*1.d0/16.d0/pi**2*(&
           4.d0*c0f(p1sq,p2sq,qsq,mW,mW,mW)&
           +(6.d0+mh**2/mW**2)*&
          (c23f(p1sq,p2sq,qsq,mW,mW,mW)+c12f(p1sq,p2sq,qsq,mW,mW,mW) )&
           )
       end function

!-----------------------------------------------------------------
       complex (kind(0d0)) function rGam2_hgamgam_SM(p1sq,p2sq,qsq)
       double precision, intent(in):: qsq,p1sq,p2sq

       rGam2_hgamgam_SM=&
            rGam2_hgamgam_F_SM(p1sq,p2sq,qsq)&
           +rGam2_hgamgam_W_SM(p1sq,p2sq,qsq)

       end function
!-----------------------------------------------------------------
       complex (kind(0d0)) function rGam1_hgamgam_SM(p1sq,p2sq,qsq)
       double precision, intent(in):: qsq,p1sq,p2sq

       rGam1_hgamgam_SM=&
            rGam1_hgamgam_F_SM(p1sq,p2sq,qsq)&
           +rGam1_hgamgam_W_SM(p1sq,p2sq,qsq)

       end function

!------------- hff vertex correction to h-->Zff  --------------------------------------
       complex (kind(0d0)) function Thff_Z_SM(vf,af,qsq,cos_the)
       double precision, intent(in):: qsq,cos_the
       double precision:: s24,s23,vf,af
       s24=1.d0/2.d0*(mh**2+mZ**2-qsq-cos_the*sqrt(mh**4+(mZ**2-qsq)**2-2.d0*mh**2*(mZ**2+qsq)))
       s23=mh**2+mz**2-qsq-s24

       Thff_Z_SM = &
            alpha_em/4.d0/pi*(&
            4.d0/(cw**2*sw**2)*(vf**4+6.d0*vf**2*af**2+af**4)*(-1.d0/2.d0)*(&
            c0f(s23,0.d0,mh**2,mZ,0.d0,mZ)+c11f(s23,0.d0,mh**2,mZ,0.d0,mZ)  )&
            +cw**2/sw**2*(vf+af)**2*(-1.d0/2.d0)*(&
            c0f(s23,0.d0,mh**2,mW,0.d0,mW)+c11f(s23,0.d0,mh**2,mW,0.d0,mW)  )&
            
            +4.d0/(cw**2*sw**2)*(vf**4+6.d0*vf**2*af**2+af**4)*(1.d0/2.d0)&
            *c12f(0.d0,s24,mh**2,mZ,0.d0,mZ)&
            +cw**2/sw**2*(vf+af)**2*(1.d0/2.d0)&
            *c12f(0.d0,s24,mh**2,mW,0.d0,mW)  &
         )

       end function

!------------- hff vertex correction to h-->Wffp  --------------------------------------
       complex (kind(0d0)) function Thff_W_SM(vf,af,qsq,cos_the)
       double precision, intent(in):: qsq,cos_the
       double precision:: s24,s23,vf,af,vfp,afp
       s24=1.d0/2.d0*(mh**2+mW**2-qsq-cos_the*sqrt(mh**4+(mW**2-qsq)**2-2.d0*mh**2*(mW**2+qsq)))
       s23=mh**2+mW**2-qsq-s24

       if (vf .eq. vt) then 
        vfp = vb ; afp = ab
       elseif (vf .eq. vb) then 
        vfp = vt ; afp = at 
       elseif (vf .eq. ve) then 
        vfp = vnu ; afp = anu
       elseif (vf .eq. vnu) then 
        vfp = ve ; afp = ae

       endif


       Thff_W_SM = &
            1.d0/cW**2*alpha_em/4.d0/pi*(&
            4.d0/(cw**2*sw**2)*(vfp+afp)**2*(-1.d0/2.d0)*(&
            c0f(s23,0.d0,mh**2,mZ,0.d0,mZ)+c11f(s23,0.d0,mh**2,mZ,0.d0,mZ)  )&
            +2.d0*cw**2/sw**2*(-1.d0/2.d0)*(&
            c0f(s23,0.d0,mh**2,mW,0.d0,mW)+c11f(s23,0.d0,mh**2,mW,0.d0,mW)  )&
            
            +4.d0/(cw**2*sw**2)*(vf+af)**2*(1.d0/2.d0)&
            *c12f(0.d0,s24,mh**2,mZ,0.d0,mZ)&
            +2.d0*cw**2/sw**2*(1.d0/2.d0)&
            *c12f(0.d0,s24,mh**2,mW,0.d0,mW)  &
         )


       end function
!------------- box diagram contributions for ff-->gamh  --------------------------------------
       complex (kind(0d0)) function F_13_photon_Bi(qsq,cos_the,k)
       double precision, intent(in):: qsq,cos_the
       double precision:: s24,s23
       integer, intent(in) :: k
       s24=1.d0/2.d0*(mh**2+mphoton**2-qsq-cos_the*sqrt(mh**4+(mphoton**2-qsq)**2-2.d0*mh**2*(mphoton**2+qsq)))
       s23=mh**2+mphoton**2-qsq-s24

       if (k .eq. 1) then 
          F_13_photon_Bi=2.d0*(d13f(0.d0,0.d0,mphoton**2,mh**2,qsq,s23,mW,0.d0,mW,mW)&
                      -d12f(0.d0,0.d0,mphoton**2,mh**2,qsq,s23,mW,0.d0,mW,mW)&
                 +2.d0*d26f(0.d0,0.d0,mphoton**2,mh**2,qsq,s23,mW,0.d0,mW,mW))

       elseif (k .eq. 2) then 
          F_13_photon_Bi=4.d0*(d0f(0.d0,0.d0,mphoton**2,mh**2,qsq,s24,mW,0.d0,mW,mW)&
                      +d11f(0.d0,0.d0,mphoton**2,mh**2,qsq,s24,mW,0.d0,mW,mW)&
                      +d13f(0.d0,0.d0,mphoton**2,mh**2,qsq,s24,mW,0.d0,mW,mW)&
                      +d25f(0.d0,0.d0,mphoton**2,mh**2,qsq,s24,mW,0.d0,mW,mW)&
                      )

       elseif (k .eq. 3) then 
          F_13_photon_Bi=-4.d0*(d12f(0.d0,0.d0,mphoton**2,mh**2,qsq,s23,mW,0.d0,mW,mW)&
                      -d13f(0.d0,0.d0,mphoton**2,mh**2,qsq,s23,mW,0.d0,mW,mW))

       elseif (k .eq. 4) then 
          F_13_photon_Bi=0.d0

       elseif (k .eq. 5) then 
          F_13_photon_Bi=-2.d0*(d0f(0.d0,mphoton**2,0.d0,mh**2,s23,s24,mW,0.d0,0.d0,mW)&
                       +d11f(0.d0,mphoton**2,0.d0,mh**2,s23,s24,mW,0.d0,0.d0,mW)&
                       +d12f(0.d0,mphoton**2,0.d0,mh**2,s23,s24,mW,0.d0,0.d0,mW)&
                       +d24f(0.d0,mphoton**2,0.d0,mh**2,s23,s24,mW,0.d0,0.d0,mW)&
                        )

       elseif (k .eq. 6) then 
          F_13_photon_Bi=-2.d0*(d0f(0.d0,mphoton**2,0.d0,mh**2,s24,s23,mZ,mmu**2,mmu**2,mZ)&
                       +d11f(0.d0,mphoton**2,0.d0,mh**2,s24,s23,mZ,mmu**2,mmu**2,mZ)&
                       +d12f(0.d0,mphoton**2,0.d0,mh**2,s24,s23,mZ,mmu**2,mmu**2,mZ)&
                       +d24f(0.d0,mphoton**2,0.d0,mh**2,s24,s23,mZ,mmu**2,mmu**2,mZ)&
                        )

       endif
       end function

!------------- box diagram contributions for h-->Zff --------------------------------------
      double complex function F_VFVV_pgam1(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)
      double precision, intent(in):: q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4

      F_VFVV_pgam1=2.d0*(d13f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
                      -d12f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
                 +2.d0*d26f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4))
          end function
!-----------------------------------------------------------------------------
      double complex function F_VFVV_pgam2(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)
      double precision, intent(in):: q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4

          F_VFVV_pgam2=4.d0*(d0f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
                      +d11f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
                      +d13f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
                      +d25f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4))

          end function
!-----------------------------------------------------------------------------
      double complex function F_VFVV_gam(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)
      double precision, intent(in):: q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4

          F_VFVV_gam=-(2.d0*c0f(q12sq,q3sq,q123sq,m1,m3,m4)&
           +4.d0*d27f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
           +(2.d0*q12sq+q13sq-q123sq)*(d0f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
                            +d11f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4) )&
           +2.d0*(q13sq-q3sq)*d12f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
           +q13sq*d13f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4) )

          end function

!-----------------------------------------------------------------------------
      double complex function F_VFVV_gam_photon(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)
      double precision, intent(in):: q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4

          F_VFVV_gam_photon=-(2.d0*c0f(q12sq,q3sq,q123sq,m1,m3,m4)&
           +4.d0*d27f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
           +(2.d0*q12sq+q13sq-q123sq+3.d0*q2sq-4.d0*q1sq)*d0f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
           +(2.d0*q12sq+q13sq-q123sq-2.d0*q2sq-4.d0*q1sq)*d11f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4) &
           +(2.d0*q13sq-2.d0*q3sq+3.d0*q2sq)*d12f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
           +(q13sq-q2sq)*d13f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4) )

          end function
!-----------------------------------------------------------------------------
      double complex function F_VFVS_pgam(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)
      double precision, intent(in):: q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4
          F_VFVS_pgam=-4.d0*(d12f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
                      -d13f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4) )

          end function
!-----------------------------------------------------------------------------
      double complex function F_VFVS_gam(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)
      double precision, intent(in):: q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4

          F_VFVS_gam=c0f(q12sq,q3sq,q123sq,m1,m3,m4)&
                     +2.d0*(-(q13sq-q123sq)*(d0f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
                               +d11f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4))&
                               +q13sq*d13f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4) )

          end function
!-----------------------------------------------------------------------------
      double complex function F_VFVS_gam_photon(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)
      double precision, intent(in):: q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4

          F_VFVS_gam_photon=c0f(q12sq,q3sq,q123sq,m1,m3,m4)&
             -(2.d0*q13sq-2.d0*q123sq-q1sq-q2sq)*d0f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
             -(2.d0*q13sq-2.d0*q123sq-q1sq)*d11f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
             +q2sq*d12f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
             +(2.d0*q13sq-q1sq-q2sq)*d13f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4) 

          end function
!-----------------------------------------------------------------------------
      double complex function F_VFFV_pgam1(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)
      double precision, intent(in):: q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4
          F_VFFV_pgam1=-2.d0*(d0f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
                       +d11f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
                       +d12f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
                       +d24f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
                        )

          end function
!-----------------------------------------------------------------------------
      double complex function F_VFFV_pgam2(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)
      double precision, intent(in):: q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4
          F_VFFV_pgam2=-2.d0*d26f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)

          end function
!-----------------------------------------------------------------------------
      double complex function F_VFFV_gam(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)
      double precision, intent(in):: q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4
          F_VFFV_gam=-(& 
               c0f(q12sq,q3sq,q123sq,m1,m3,m4)&
              -2.d0*d27f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
              +(q12sq-q2sq)*(d0f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)&
              +d11f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4) )&
              +q2sq*d12f(q1sq,q2sq,q3sq,q123sq,q12sq,q13sq,m1,m2,m3,m4)& 
              )                  

          end function

!-----------------------------------------------------------------------------
       complex (kind(0d0)) function F_13_BZi(qsq,cos_the,k)
       double precision, intent(in):: qsq,cos_the
       double precision:: s24,s23
       integer, intent(in) :: k
       s24=1.d0/2.d0*(mh**2+mZ**2-qsq-cos_the*sqrt(mh**4+(mZ**2-qsq)**2-2.d0*mh**2*(mZ**2+qsq)))
       s23=mh**2+mz**2-qsq-s24

       if (k .eq. 1) then 
          F_13_BZi=F_VFVV_pgam1(0.d0,0.d0,mZ**2,mh**2,qsq,s23,mW,0.d0,mW,mW)
       elseif (k .eq. 2) then 
          F_13_BZi=F_VFVV_pgam2(0.d0,0.d0,mZ**2,mh**2,qsq,s24,mW,0.d0,mW,mW)
       elseif (k .eq. 3) then 
          F_13_BZi=F_VFVS_pgam(0.d0,0.d0,mZ**2,mh**2,qsq,s23,mW,0.d0,mW,mW)
       elseif (k .eq. 4) then 
          F_13_BZi=0.d0
       elseif (k .eq. 5) then 
          F_13_BZi=F_VFFV_pgam1(0.d0,mZ**2,0.d0,mh**2,s23,s24,mW,0.d0,0.d0,mW)
       elseif (k .eq. 6) then 
          F_13_BZi=F_VFFV_pgam1(0.d0,mZ**2,0.d0,mh**2,s23,s24,mZ,0.d0,0.d0,mZ)
       endif
       end function
!-----------------------------------------------------------------------------
       complex (kind(0d0)) function F_14_BZi(qsq,cos_the,k)
       double precision, intent(in):: qsq,cos_the
       double precision:: s24,s23
       integer, intent(in) :: k
       s24=1.d0/2.d0*(mh**2+mZ**2-qsq-cos_the*sqrt(mh**4+(mZ**2-qsq)**2-2.d0*mh**2*(mZ**2+qsq)))
       s23=mh**2+mz**2-qsq-s24

       if (k .eq. 1) then 
          F_14_BZi=F_VFVV_pgam2(0.d0,0.d0,mZ**2,mh**2,qsq,s23,mW,0.d0,mW,mW)
       elseif (k .eq. 2) then 
          F_14_BZi=F_VFVV_pgam1(0.d0,0.d0,mZ**2,mh**2,qsq,s24,mW,0.d0,mW,mW)
       elseif (k .eq. 3) then 
          F_14_BZi=0.d0
       elseif (k .eq. 4) then 
          F_14_BZi=F_VFVS_pgam(0.d0,0.d0,mZ**2,mh**2,qsq,s24,mW,0.d0,mW,mW)
       elseif (k .eq. 5) then 
          F_14_BZi=F_VFFV_pgam2(0.d0,mZ**2,0.d0,mh**2,s23,s24,mW,0.d0,0.d0,mW)
       elseif (k .eq. 6) then 
          F_14_BZi=F_VFFV_pgam2(0.d0,mZ**2,0.d0,mh**2,s23,s24,mZ,0.d0,0.d0,mZ)
       endif
       end function
!-----------------------------------------------------------------------------
       complex (kind(0d0)) function F_gam_BZi(qsq,cos_the,k)
       double precision, intent(in):: qsq,cos_the
       double precision:: s24,s23
       integer, intent(in) :: k
       s24=1.d0/2.d0*(mh**2+mZ**2-qsq-cos_the*sqrt(mh**4+(mZ**2-qsq)**2-2.d0*mh**2*(mZ**2+qsq)))
       s23=mh**2+mz**2-qsq-s24

       if (k .eq. 1) then 
          F_gam_BZi=F_VFVV_gam(0.d0,0.d0,mZ**2,mh**2,qsq,s23,mW,0.d0,mW,mW)
       elseif (k .eq. 2) then 
          F_gam_BZi=F_VFVV_gam(0.d0,0.d0,mZ**2,mh**2,qsq,s24,mW,0.d0,mW,mW)
       elseif (k .eq. 3) then 
          F_gam_BZi=F_VFVS_gam(0.d0,0.d0,mZ**2,mh**2,qsq,s23,mW,0.d0,mW,mW)
       elseif (k .eq. 4) then 
          F_gam_BZi=F_VFVS_gam(0.d0,0.d0,mZ**2,mh**2,qsq,s24,mW,0.d0,mW,mW)
       elseif (k .eq. 5) then 
          F_gam_BZi=F_VFFV_gam(0.d0,mZ**2,0.d0,mh**2,s23,s24,mW,0.d0,0.d0,mW)
       elseif (k .eq. 6) then 
          F_gam_BZi=F_VFFV_gam(0.d0,mZ**2,0.d0,mh**2,s23,s24,mZ,0.d0,0.d0,mZ)

       endif
       end function
!-----------------------------------------------------------------------------
       complex (kind(0d0)) function B_BZi(qsq,cos_the,k)
       double precision, intent(in):: qsq,cos_the
       double precision:: s24,s23
       integer, intent(in) :: k

       s24=1.d0/2.d0*(mh**2+mZ**2-qsq-cos_the*sqrt(mh**4+(mZ**2-qsq)**2-2.d0*mh**2*(mZ**2+qsq)))
       s23=mh**2+mz**2-qsq-s24

        B_BZi=qsq*F_gam_BZi(qsq,cos_the,k)&
             +(s23*s24-mZ**2*mh**2)/4.d0/mZ**2*(&
             2.d0*F_gam_BZi(qsq,cos_the,k)&
             -(s23-mZ**2)*F_13_BZi(qsq,cos_the,k)&
             -(s24-mZ**2)*F_14_BZi(qsq,cos_the,k)& 
              )

       end function
!-----------------------------------------------------------------------------
       complex (kind(0d0)) function B_Z_box(vf,af,qsq,cos_the)
       double precision, intent(in):: qsq,cos_the,vf,af
       double precision:: s24,s23,vfp,afp


       s24=1.d0/2.d0*(mh**2+mZ**2-qsq-cos_the*sqrt(mh**4+(mZ**2-qsq)**2-2.d0*mh**2*(mZ**2+qsq)))
       s23=mh**2+mz**2-qsq-s24

       if (vf .eq. vt) then 
        vfp = vb ; afp = ab
       elseif (vf .eq. vb) then 
        vfp = vt ; afp = at 
       elseif (vf .eq. ve) then 
        vfp = vnu ; afp = anu
       elseif (vf .eq. vnu) then 
        vfp = ve ; afp = ae

       endif


       B_Z_box=alpha_EM/4.d0/pi*(&
            +cW**2/sW**2*(vf+af)*(vfp+afp)*(-2.d0*B_BZi(qsq,cos_the,5))&
            +4.d0/cW**2/sW**2*(vf**4+6.d0*vf**2*af**2+af**4)*(-2.d0*B_BZi(qsq,cos_the,6))&
            +cW**2*af*(vf+af)*(&
                cW**2/sW**2*(-4.d0*B_BZi(qsq,cos_the,1)&
                             -4.d0*B_BZi(qsq,cos_the,2)&
                             )&
                +2.d0*B_BZi(qsq,cos_the,3)+2.d0*B_BZi(qsq,cos_the,4) &
                              )&
                              )
               end function
!------------- box diagram contributions for h-->Wffp --------------------------------------
       complex (kind(0d0)) function F_13_BWi(qsq,cos_the,k,mf)
       double precision, intent(in):: qsq,cos_the,mf
       double precision:: s24,s23,mfp
       integer, intent(in) :: k

        if (mf .eq. mc) then 
        mfp = msir
        elseif (mf .eq. msir) then 
        mfp = mc
        elseif (mf .eq. mtau) then 
        mfp = 0.d0
        elseif (mf .eq. mmu) then 
        mfp = 0.d0
        endif


! expressinos in mf=mfp=0
!        s24=1.d0/2.d0*(mh**2+mW**2-qsq-cos_the*sqrt(mh**4+(mW**2-qsq)**2-2.d0*mh**2*(mW**2+qsq)))
!        s23=mh**2+mW**2-qsq-s24

! expressinos in nonzero mf,mfp 
         s24=1.d0/(2.d0*qsq)*(&
             -(mf**2-mfp**2)*(mh**2-mW**2)+(mf**2+mfp**2+mW**2+mh**2)*qsq-qsq**2+cos_the&
             *sqrt(mh**4+(mW**2-qsq)**2-2.d0*mh**2*(mW**2+qsq))&
             *sqrt(mf**4+(mfp**2-qsq)**2-2.d0*mf**2*(mfp**2+qsq)) )
        s23=mh**2+mW**2+mf**2+mfp**2-qsq-s24




       if (k .eq. 1) then 
          F_13_BWi=F_VFVV_pgam1(mfp**2,mf**2,mW**2,mh**2,qsq,s23,mZ,mfp,mW,mZ)
       elseif (k .eq. 2) then 
          F_13_BWi=F_VFVV_pgam2(mf**2,mfp**2,mW**2,mh**2,qsq,s24,mZ,mf,mW,mZ)
       elseif (k .eq. 3) then 
          F_13_BWi=F_VFVV_pgam1(mfp**2,mf**2,mW**2,mh**2,qsq,s23,mW,mf,mZ,mW)
       elseif (k .eq. 4) then 
          F_13_BWi=F_VFVV_pgam2(mf**2,mfp**2,mW**2,mh**2,qsq,s24,mW,mfp,mZ,mW)
       elseif (k .eq. 5) then 
          F_13_BWi=F_VFVV_pgam1(mfp**2,mf**2,mW**2,mh**2,qsq,s23,mW,mf,mphoton,mW)
       elseif (k .eq. 6) then 
          F_13_BWi=F_VFVV_pgam2(mf**2,mfp**2,mW**2,mh**2,qsq,s24,mW,mfp,mphoton,mW)
       elseif (k .eq. 7) then 
          F_13_BWi=F_VFVS_pgam(mfp**2,mf**2,mW**2,mh**2,qsq,s23,mW,mf,mZ,mw)
       elseif (k .eq. 8) then 
          F_13_BWi=0.d0
       elseif (k .eq. 9) then 
          F_13_BWi=F_VFVS_pgam(mfp**2,mf**2,mW**2,mh**2,qsq,s23,mW,mf,mphoton,mw)
       elseif (k .eq. 10) then 
          F_13_BWi=0.d0

       elseif (k .eq. 12) then 
          F_13_BWi=F_VFFV_pgam1(mf**2,mW**2,mfp**2,mh**2,s23,s24,mZ,mf,mfp,mZ)
       endif
       end function
!-----------------------------------------------------------------------------
       complex (kind(0d0)) function F_14_BWi(qsq,cos_the,k,mf)
       double precision, intent(in):: qsq,cos_the,mf
       double precision:: s24,s23,mfp
       integer, intent(in) :: k

        if (mf .eq. mc) then 
        mfp = msir
        elseif (mf .eq. msir) then 
        mfp = mc
        elseif (mf .eq. mtau) then 
        mfp = 0.d0
        elseif (mf .eq. mmu) then 
        mfp = 0.d0
        endif

! expressinos in mf=mfp=0
!        s24=1.d0/2.d0*(mh**2+mW**2-qsq-cos_the*sqrt(mh**4+(mW**2-qsq)**2-2.d0*mh**2*(mW**2+qsq)))
!        s23=mh**2+mW**2-qsq-s24

! expressinos in nonzero mf,mfp 
         s24=1.d0/(2.d0*qsq)*(&
             -(mf**2-mfp**2)*(mh**2-mW**2)+(mf**2+mfp**2+mW**2+mh**2)*qsq-qsq**2+cos_the&
             *sqrt(mh**4+(mW**2-qsq)**2-2.d0*mh**2*(mW**2+qsq))&
             *sqrt(mf**4+(mfp**2-qsq)**2-2.d0*mf**2*(mfp**2+qsq)) )

        s23=mh**2+mW**2+mf**2+mfp**2-qsq-s24



       if (k .eq. 1) then 
          F_14_BWi=F_VFVV_pgam2(mfp**2,mf**2,mW**2,mh**2,qsq,s23,mZ,mfp,mW,mZ)
       elseif (k .eq. 2) then 
          F_14_BWi=F_VFVV_pgam1(mf**2,mfp**2,mW**2,mh**2,qsq,s24,mZ,mf,mW,mZ)
       elseif (k .eq. 3) then 
          F_14_BWi=F_VFVV_pgam2(mfp**2,mf**2,mW**2,mh**2,qsq,s23,mW,mf,mZ,mW)
       elseif (k .eq. 4) then 
          F_14_BWi=F_VFVV_pgam1(mf**2,mfp**2,mW**2,mh**2,qsq,s24,mW,mfp,mZ,mW)
       elseif (k .eq. 5) then 
          F_14_BWi=F_VFVV_pgam2(mfp**2,mf**2,mW**2,mh**2,qsq,s23,mW,mf,mphoton,mW)
       elseif (k .eq. 6) then 
          F_14_BWi=F_VFVV_pgam1(mf**2,mfp**2,mW**2,mh**2,qsq,s24,mW,mfp,mphoton,mW)
       elseif (k .eq. 7) then 
          F_14_BWi=0.d0
       elseif (k .eq. 8) then 
          F_14_BWi=F_VFVS_pgam(mf**2,mfp**2,mW**2,mh**2,qsq,s24,mW,mfp,mZ,mw)
       elseif (k .eq. 9) then 
          F_14_BWi=0.d0
       elseif (k .eq. 10) then 
          F_14_BWi=F_VFVS_pgam(mf**2,mfp**2,mW**2,mh**2,qsq,s24,mW,mfp,mphoton,mw)

       elseif (k .eq. 12) then 
          F_14_BWi=F_VFFV_pgam2(mf**2,mW**2,mfp**2,mh**2,s23,s24,mZ,mf,mfp,mZ)
       endif
       end function
!-----------------------------------------------------------------------------
       complex (kind(0d0)) function F_gam_BWi(qsq,cos_the,k,mf)
       double precision, intent(in):: qsq,cos_the,mf
       integer, intent(in) :: k
       double precision:: mfp,Qf,Qfp,delE,E2,E3,E4,absp2,s23,s24,q340,beta2,betabar,gambar
       double precision:: E3hat,E4hat,beta34,p3p4,p3p4_2,p2p4,p2p3
       double precision:: p1(4),p2(4),p3(4),p4(4),alpha34bar,alpha23bar,alpha24bar
       double precision:: mgam

       mgam = mphoton

        if (mf .eq. mc) then 
        mfp = msir
        elseif (mf .eq. msir) then 
        mfp = mc
        elseif (mf .eq. mtau) then 
        mfp = 0.d0
        elseif (mf .eq. mmu) then 
        mfp = 0.d0
        endif



       q340=mh/2.d0*(1.d0-mW**2/mh**2+qsq/mh**2)
       beta2=sqrt(1.d0-2.d0*(mW**2/mh**2+qsq/mh**2)+(mW**2/mh**2-qsq/mh**2)**2)
       betabar=-mh/(2.d0*q340)*beta2
       gambar=q340/sqrt(qsq)
       beta34=sqrt(1.d0-2.d0*(mf**2/qsq+mfp**2/qsq)+(mf**2/qsq-mfp**2/qsq)**2)
       absp2=mh/2.d0*beta2

       E2=mh/2.d0*(1.d0+mW**2/mh**2-qsq/mh**2)
       E3hat= sqrt(qsq)/2.d0*(1.d0+mf**2/qsq-mfp**2/qsq)
       E4hat= sqrt(qsq)/2.d0*(1.d0-mf**2/qsq+mfp**2/qsq)

       p1(1) = mh; p1(2) = 0.d0; p1(3) = 0.d0; p1(4) = 0.d0;
       p2(1) = E2; p2(2) = 0.d0; p2(3) = 0.d0; p2(4) = absp2;
       p3(1) = E3hat*gambar+1.d0/2.d0*sqrt(qsq)*beta34*betabar*gambar*cos_the
       p3(2) = 1.d0/2.d0*sqrt(qsq)*beta34*sqrt(1.d0-cos_the**2)
       p3(3) = 0.d0
       p3(4) = E3hat*betabar*gambar+1.d0/2.d0*sqrt(qsq)*beta34*gambar*cos_the
       p4(1) = E4hat*gambar-1.d0/2.d0*sqrt(qsq)*beta34*betabar*gambar*cos_the
       p4(2) = -1.d0/2.d0*sqrt(qsq)*beta34*sqrt(1.d0-cos_the**2)
       p4(3) = 0.d0
       p4(4) = E4hat*betabar*gambar-1.d0/2.d0*sqrt(qsq)*beta34*gambar*cos_the

       p3p4 = p3(1)*p4(1)-p3(2)*p4(2)-p3(3)*p4(3)-p3(4)*p4(4)
       p2p3 = p3(1)*p2(1)-p3(2)*p2(2)-p3(3)*p2(3)-p3(4)*p2(4)
       p2p4 = p4(1)*p2(1)-p4(2)*p2(2)-p4(3)*p2(3)-p4(4)*p2(4)
       s23 = mW**2+2.d0*p2p3+mf**2 
       s24 = mW**2+2.d0*p2p4+mfp**2




       if (k .eq. 1) then 
          F_gam_BWi=F_VFVV_gam(mfp**2,mf**2,mW**2,mh**2,qsq,s23,mZ,mfp,mW,mZ)
       elseif (k .eq. 2) then 
          F_gam_BWi=F_VFVV_gam(mf**2,mfp**2,mW**2,mh**2,qsq,s24,mZ,mf,mW,mZ)
       elseif (k .eq. 3) then 
          F_gam_BWi=F_VFVV_gam(mfp**2,mf**2,mW**2,mh**2,qsq,s23,mW,mf,mZ,mW)
       elseif (k .eq. 4) then 
          F_gam_BWi=F_VFVV_gam(mf**2,mfp**2,mW**2,mh**2,qsq,s24,mW,mfp,mZ,mW)
       elseif (k .eq. 5) then 
!
          F_gam_BWi=F_VFVV_gam_photon(mfp**2,mf**2,mW**2,mh**2,qsq,s23,mW,mf,mgam,mW)
       elseif (k .eq. 6) then 
!
          F_gam_BWi=F_VFVV_gam_photon(mf**2,mfp**2,mW**2,mh**2,qsq,s24,mW,mfp,mgam,mW)
       elseif (k .eq. 7) then 
          F_gam_BWi=F_VFVS_gam(mfp**2,mf**2,mW**2,mh**2,qsq,s23,mW,mf,mZ,mw)
       elseif (k .eq. 8) then 
          F_gam_BWi=F_VFVS_gam(mf**2,mfp**2,mW**2,mh**2,qsq,s24,mW,mfp,mZ,mw)
       elseif (k .eq. 9) then 
!
          F_gam_BWi=F_VFVS_gam_photon(mfp**2,mf**2,mW**2,mh**2,qsq,s23,mW,mf,mgam,mw)
       elseif (k .eq. 10) then 
!
          F_gam_BWi=F_VFVS_gam_photon(mf**2,mfp**2,mW**2,mh**2,qsq,s24,mW,mfp,mgam,mw)

       elseif (k .eq. 12) then 
          F_gam_BWi=F_VFFV_gam(mf**2,mW**2,mfp**2,mh**2,s23,s24,mZ,mf,mfp,mZ)
       endif
       end function
!-----------------------------------------------------------------------------
       complex (kind(0d0)) function B_BWi(qsq,cos_the,k,mf)
       double precision, intent(in):: qsq,cos_the,mf
       double precision:: s24,s23,mfp,epsW,epsQ,x1,x2
       integer, intent(in) :: k

        if (mf .eq. mc) then 
        mfp = msir
        elseif (mf .eq. msir) then 
        mfp = mc
        elseif (mf .eq. mtau) then 
        mfp = 0.d0
        elseif (mf .eq. mmu) then 
        mfp = 0.d0
        endif

! expressinos in mf=mfp=0
! we have used exactly same valeables in the function named dGam_dqsqdcos_hWW_SM_NLO.If we do not so, we are suffer from
! bit numerical diffrence  in the check of IR div. It comes from the fact that canleations of kinetic factor "ckin1" does
! not subtlely work in box contribuions in dGam_dqsqdcos_hWW_SM_NLO. Hence we should use expressions in mf=mfp=0 here.
       epsW=mW**2/mh**2
       epsQ=qsq/mh**2
       x1=1.d0+epsW-epsQ
       x2=1.d0/2.d0*(2.d0-x1+sqrt(x1**2-4.d0*mW**2/mh**2)*cos_the)
       s24=mh**2*(1.d0-x2)
       s23=mh**2+mW**2-qsq-s24

! expressinos in nonzero mf,mfp 
!          s24=1.d0/(2.d0*qsq)*(&
!              -(mf**2-mfp**2)*(mh**2-mW**2)+(mf**2+mfp**2+mW**2+mh**2)*qsq-qsq**2+cos_the&
!              *sqrt(mh**4+(mW**2-qsq)**2-2.d0*mh**2*(mW**2+qsq))&
!              *sqrt(mf**4+(mfp**2-qsq)**2-2.d0*mf**2*(mfp**2+qsq)) )
!         s23=mh**2+mW**2+mf**2+mfp**2-qsq-s24

        B_BWi=&
              qsq*F_gam_BWi(qsq,cos_the,k,mf)&
              +(s23*s24-mW**2*mh**2)/4.d0/mW**2*(&
              2.d0*F_gam_BWi(qsq,cos_the,k,mf)&
              -(s23-mW**2)*F_13_BWi(qsq,cos_the,k,mf)&
              -(s24-mW**2)*F_14_BWi(qsq,cos_the,k,mf)& 
               )

       end function
!-----------------------------------------------------------------------------
       complex (kind(0d0)) function B_W_box(qsq,cos_the,mf)
       double precision, intent(in):: qsq,cos_the,mf
       double precision:: s24,s23,vfp,afp,Qf,Qfp,vf,af

        if (mf .eq. mc) then 
        vf = vt ; af = at ; Qf = Qt
        vfp = vb ; afp = ab ; Qfp = Qb
        elseif (mf .eq. msir) then 
        vf = vb ; af = ab ; Qf = Qb
        vfp  = vt ; afp  = at ; Qfp  = Qt
        elseif (mf .eq. mtau) then 
        vf = ve ; af = ae ; Qf = Qe
        vfp = vnu ; afp = anu ; Qfp = 0.d0
        elseif (mf .eq. mmu) then 
        vf = ve ; af = ae ; Qf = Qe 
        vfp = vnu ; afp = anu ; Qfp = 0.d0
        endif


       B_W_box=-4.d0*alpha_EM/4.d0/pi*(&
             afp*(vfp+afp)*(4.d0/sW**2/cW**2*B_BWi(qsq,cos_the,1,mf)&
                             +4.d0/sW**2*B_BWi(qsq,cos_the,4,mf)&
                            +2.d0/cW**2*B_BWi(qsq,cos_the,8,mf) &
              )&
            +af*(vf+af)*(4.d0/sW**2/cW**2*B_BWi(qsq,cos_the,2,mf)&
                             +4.d0/sW**2*B_BWi(qsq,cos_the,3,mf)&
                            +2.d0/cW**2*B_BWi(qsq,cos_the,7,mf) &
              )&          
            +afp*Qfp*(4.d0*B_BWi(qsq,cos_the,6,mf)-2.d0*B_BWi(qsq,cos_the,10,mf) )&
            +af*Qf*(4.d0*B_BWi(qsq,cos_the,5,mf)-2.d0*B_BWi(qsq,cos_the,9,mf) )&

          +(vf+af)*(vfp+afp)*2.d0/sW**2/cW**2*B_BWi(qsq,cos_the,12,mf)&
                              )
               end function
!---------------------------------------------------------------------------------------
       double precision function cos_the_f(qsq)
       double precision, intent(in):: qsq
       cos_the_f=1.d0-0.000000001d0
!       cos_the_f=1.d0
       end function
       double precision function cos_the_i(qsq)
       double precision, intent(in):: qsq
       cos_the_i=-1.d0+0.000000001d0
!       cos_the_i=-1.d0
       end function


!------------------ dGamma( h-->Zff )/dqsq  at NLO ------------------

       double precision function dGam_dqsq_hZZ_SM(qsq,vf,af,Iff,Qf,OR)
         double precision, intent(in):: qsq,vf,af,Iff,Qf
         character(*), intent(in):: OR
         double precision:: epsZ,epsQ,x1,ckin_a,ckin_b
         epsZ=mZ**2/mh**2
         epsQ=qsq/mh**2
         x1=1.d0+epsZ-epsQ
         ckin_a=sqrt(x1**2-4.d0*epsZ)*((x1-6.d0*epsZ)**2+8.d0*epsZ*(1.d0-3.d0*epsZ))
         ckin_b=(x1**2-4.d0*epsZ)**(3.d0/2.d0)*(x1-2.d0*epsZ)*mh**2/2.d0

         if (OR .eq. 'NLOEW') then
            ! note: this calcration is not included contributuion from hff vertex corrections and box diagrams as well as QED corrections
            dGam_dqsq_hZZ_SM= &
                 mh**3*gz**4/384.d0/pi**3*(vf**2+af**2)/(qsq-mz**2)**2*ckin_a*(&
                 1.d0/2.d0&                                                                   !Tree cont.
                 -(Del_r_SM())&                                                         !Delta_r
                 -1.d0/2.d0*real(rdPiZZ_SM(mZ**2))&                                       !rdPiZZ
                 +1.d0/(2.d0*mZ**2/vev)*(&
                 +real(Gam1_hZZ_loop_SM(mZ**2,qsq,mh**2))&                           !hZZ Gam1 NLO cont.
                 +ckin_b/ckin_a*real(rGam2_hZZ_SM(mZ**2,qsq,mh**2))/mZ**2 )&         !hZZ Gam2 NLO cont.
                 -real(rPiZZ_SM(qsq))/(qsq-mZ**2)&                                  ! PiZZ cont.
                 +1.d0/(vf**2+af**2)*(&
                 (qsq-mZ**2)/(qsq)*vf*Qf*cw*sw*(&
                 1.d0/(2.d0*mZ**2/vev)*(real(rGam1_hZgam_SM(mZ**2,qsq,mh**2))& !hZGam Gam1
                 +ckin_b/ckin_a*real(rGam2_hZgam_SM(mZ**2,qsq,mh**2)) )&       !hZGam Gam2
                 -real(rPiZG_SM(qsq))/(qsq-mZ**2)&                             ! PiZG cont.
                 )&
                 -1.d0/gz*(&
                 +vf*real(rGamV_Zff_loop_SM(qsq,vf,af,Iff))&             ! ZffV NLO cont.
                 -af*real(rGamA_Zff_loop_SM(qsq,vf,af,Iff))&              ! ZffA NLO cont.
                 )&
                 )&
                 )
         else if (OR .eq. 'NLOQCD') then
            if (Qf .eq. Qt) then
               dGam_dqsq_hZZ_SM= &
                    mh**3*gz**4/384.d0/pi**3*(vf**2+af**2)/(qsq-mz**2)**2*ckin_a*&
                    1.d0/2.d0*(1.d0+alphas(lam_QCD4,mh,4.d0)/pi)
            else if (Qf .eq. Qb) then
               dGam_dqsq_hZZ_SM= &
                    mh**3*gz**4/384.d0/pi**3*(vf**2+af**2)/(qsq-mz**2)**2*ckin_a*&
                    1.d0/2.d0*(1.d0+alphas(lam_QCD5,mh,5.d0)/pi)
            else
               dGam_dqsq_hZZ_SM=0.d0
            end if
         else if (OR .eq. 'LO') then
            dGam_dqsq_hZZ_SM= &
                 mh**3*gz**4/384.d0/pi**3*(vf**2+af**2)/(qsq-mz**2)**2*ckin_a*&
                 1.d0/2.d0
         else
            dGam_dqsq_hZZ_SM= 0.d0
         end if
       end function dGam_dqsq_hZZ_SM
!------------------ dGamma( h-->Zff )/dqsqdcos  at NLO ------------------

       double precision function dGam_dqsqdcos_hZZ_SM_NLO(qsq,cos_the,vf,af,Iff,Qf)
       double precision, intent(in):: qsq,cos_the,vf,af,Iff,Qf
       double precision:: epsZ,epsQ,x1,x2,ckin1,ckin2,s24,s23
       double precision:: mf,a,getlambda,mgam

       epsZ=mZ**2/mh**2
       epsQ=qsq/mh**2
       x1=1.d0+epsZ-epsQ
       x2=1.d0/2.d0*(2.d0-x1+sqrt(x1**2-4.d0*mZ**2/mh**2)*cos_the)
       s24=mh**2*(1.d0-x2)
       s23=mh**2+mZ**2-qsq-s24
       ckin1=4.d0*qsq+2.d0/mZ**2*(s24*s23-mh**2*mZ**2)
       ckin2=1.d0/mZ**2*(s24*s23-mh**2*mZ**2)*(mh**2-mZ**2-qsq)


       dGam_dqsqdcos_hZZ_SM_NLO= &
            1.d0/(256.d0*pi**3*mh)*&
            8.d0*(mZ**4/vev**2)*gz**2*(vf**2+af**2)&
            *(1.d0/2.d0)*sqrt(x1**2-4.d0*epsZ)/(qsq-mZ**2)**2*ckin1*(&
                    +1.d0/(vf**2+af**2)*(&
                +(qsq-mZ**2)*Thff_Z_SM(vf,af,qsq,cos_the)&                 !hff NLO cont.  
                  +(qsq-mZ**2)*B_Z_box(vf,af,qsq,cos_the)/ckin1&           !Box NLO cont.  
                        )&
                +Qf**2*(1.d0/2.d0)*3.d0/4.d0/pi*alpha_em&                  !QED correction
                 )

! full expressions
!        dGam_dqsqdcos_hZZ_SM_NLO= &
!             1.d0/(256.d0*pi**3*mh)*&
!             8.d0*(mZ**4/vev**2)*gz**2*(vf**2+af**2)&
!             *(1.d0/2.d0)*sqrt(x1**2-4.d0*epsZ)/(qsq-mZ**2)**2*ckin1*(&
!                  1.d0/2.d0&                                                              !Tree cont.
!                  -real(Del_r_SM())&                                                      !Delta_r
!                   -1.d0/2.d0*real(rdPiZZ_SM(mZ**2))&                                     !rdPiZZ
!                     +1.d0/(2.d0*mZ**2/vev)*(&
!                     +real(Gam1_hZZ_loop_SM(mZ**2,qsq,mh**2))&                            !hZZ Gam1 NLO cont.
!                     +ckin2/ckin1*real(rGam2_hZZ_SM(mZ**2,qsq,mh**2))/mZ**2)&             !hZZ Gam2 NLO cont.
!                      -real(rPiZZ_SM(qsq))/(qsq-mZ**2)&                                   !PiZZ cont.
!                     +1.d0/(vf**2+af**2)*(&
!                          (qsq-mZ**2)/(qsq)*vf*Qf*cw*sw*(&
!                        1.d0/(2.d0*mZ**2/vev)*(real(rGam1_hZgam_SM(mZ**2,qsq,mh**2))&     !hZGam Gam1 
!                        +ckin2/ckin1*real(rGam2_hZgam_SM(mZ**2,qsq,mh**2)) )&             !hZGam Gam2
!                        -real(rPiZG_SM(qsq))/(qsq-mZ**2)&                                 !PiZG cont.
!                      )&
!                    -1.d0/(gz)*(&
!                           +vf*real(rGamV_Zff_loop_SM(qsq,vf,af,Iff))&                    !ZffV NLO cont.
!                          -af*real(rGamA_Zff_loop_SM(qsq,vf,af,Iff))&                     !ZffA NLO cont.
!                                            )&
!                 +(qsq-mZ**2)*Thff_Z_SM(vf,af,qsq,cos_the)&                               !hff NLO cont.  
!                   +(qsq-mZ**2)*B_Z_box(vf,af,qsq,cos_the)/ckin1&                         !Box NLO cont.  
!                         )&
!          +Qf**2*(1.d0/2.d0)*3.d0/4.d0/pi*alpha_em&                                       !QED correction
!                  )

       end function
! dqsq ------------------------------------------------------------------
! [LO + part of NLO EW] ------------------------------------------------
       double precision function dGam_dqsq_hZneuneu_SM_NLOEW(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hZneuneu_SM_NLOEW= dGam_dqsq_hZZ_SM(qsq,vnu,anu,It,0.d0,'NLOEW')
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsq_hZee_SM_NLOEW(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hZee_SM_NLOEW= dGam_dqsq_hZZ_SM(qsq,ve,ae,Ie,qe,'NLOEW')
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsq_hZuu_SM_NLOEW(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hZuu_SM_NLOEW= dGam_dqsq_hZZ_SM(qsq,vt,at,It,qt,'NLOEW')
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsq_hZdd_SM_NLOEW(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hZdd_SM_NLOEW= dGam_dqsq_hZZ_SM(qsq,vb,ab,Ib,qb,'NLOEW')
       end function
![LO + NLO QCD]--------------------------------------------------------
       double precision function dGam_dqsq_hZuu_SM_NLOQCD(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hZuu_SM_NLOQCD= dGam_dqsq_hZZ_SM(qsq,vt,at,It,qt,'NLOQCD')
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsq_hZdd_SM_NLOQCD(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hZdd_SM_NLOQCD= dGam_dqsq_hZZ_SM(qsq,vb,ab,Ib,qb,'NLOQCD')
       end function
! [LO] ---------------------------------------------------------------
       double precision function dGam_dqsq_hZneuneu_SM_LO(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hZneuneu_SM_LO= dGam_dqsq_hZZ_SM(qsq,vnu,anu,It,0.d0,'LO')
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsq_hZee_SM_LO(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hZee_SM_LO= dGam_dqsq_hZZ_SM(qsq,ve,ae,Ie,qe,'LO')
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsq_hZuu_SM_LO(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hZuu_SM_LO= dGam_dqsq_hZZ_SM(qsq,vt,at,It,qt,'LO')
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsq_hZdd_SM_LO(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hZdd_SM_LO= dGam_dqsq_hZZ_SM(qsq,vb,ab,Ib,qb,'LO')
       end function
! dqsqdcos ------------------------------------------------------------------
! [part of NLO EW] ----------------------------------------------------------
       double precision function dGam_dqsqdcos_hZneuneu_SM_NLOEW(qsq,cos_the)
       double precision, intent(in):: qsq,cos_the
       dGam_dqsqdcos_hZneuneu_SM_NLOEW= &
       dGam_dqsqdcos_hZZ_SM_NLO(qsq,cos_the,vnu,anu,It,0.d0)
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsqdcos_hZee_SM_NLOEW(qsq,cos_the)
       double precision, intent(in):: qsq,cos_the
       dGam_dqsqdcos_hZee_SM_NLOEW= dGam_dqsqdcos_hZZ_SM_NLO(qsq,cos_the,ve,ae,Ie,qe)
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsqdcos_hZuu_SM_NLOEW(qsq,cos_the)
       double precision, intent(in):: qsq,cos_the
       dGam_dqsqdcos_hZuu_SM_NLOEW= &
              +dGam_dqsqdcos_hZZ_SM_NLO(qsq,cos_the,vt,at,It,qt)
        end function
!------------------------------------------------------------------
       double precision function dGam_dqsqdcos_hZdd_SM_NLOEW(qsq,cos_the)
       double precision, intent(in):: qsq,cos_the
       dGam_dqsqdcos_hZdd_SM_NLOEW= &
               +dGam_dqsqdcos_hZZ_SM_NLO(qsq,cos_the,vb,ab,Ib,qb)
       end function
! [part of NLO EW+ NLO QCD] -----------------------------------------------

       double precision function dGam_dqsqdcos_hZuu_SM_NLOEWQCD(qsq,cos_the)
       double precision, intent(in):: qsq,cos_the
       double precision:: epsZ,epsQ,x1,x2,s24,s23,ckin1
       epsZ=mZ**2/mh**2; epsQ=qsq/mh**2
       x1=1.d0+epsZ-epsQ;x2=1.d0/2.d0*(2.d0-x1+sqrt(x1**2-4.d0*mZ**2/mh**2)*cos_the)
       s24=mh**2*(1.d0-x2); s23=mh**2+mZ**2-qsq-s24
       ckin1=4.d0*qsq+2.d0/mZ**2*(s24*s23-mh**2*mZ**2)

       dGam_dqsqdcos_hZuu_SM_NLOEWQCD= &
              +dGam_dqsqdcos_hZZ_SM_NLO(qsq,cos_the,vt,at,It,qt)&
              ! NLO QCD correction
              +1.d0/(256.d0*pi**3*mh)*&
              8.d0*(mZ**4/vev**2)*gz**2*(vt**2+at**2)&
              *(1.d0/2.d0)*sqrt(x1**2-4.d0*epsZ)/(qsq-mZ**2)**2*ckin1&
              *1.d0/2.d0*alphas(lam_QCD4,mh,4.d0)/pi            
            
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsqdcos_hZdd_SM_NLOEWQCD(qsq,cos_the)
       double precision, intent(in):: qsq,cos_the
       double precision:: epsZ,epsQ,x1,x2,s24,s23,ckin1
       epsZ=mZ**2/mh**2; epsQ=qsq/mh**2
       x1=1.d0+epsZ-epsQ;x2=1.d0/2.d0*(2.d0-x1+sqrt(x1**2-4.d0*mZ**2/mh**2)*cos_the)
       s24=mh**2*(1.d0-x2);s23=mh**2+mZ**2-qsq-s24
       ckin1=4.d0*qsq+2.d0/mZ**2*(s24*s23-mh**2*mZ**2)

       dGam_dqsqdcos_hZdd_SM_NLOEWQCD= &
               +dGam_dqsqdcos_hZZ_SM_NLO(qsq,cos_the,vb,ab,Ib,qb)&
                ! QCD correction
               +1.d0/(256.d0*pi**3*mh)*&
               8.d0*(mZ**4/vev**2)*gz**2*(vb**2+ab**2)&
               *(1.d0/2.d0)*sqrt(x1**2-4.d0*epsZ)/(qsq-mZ**2)**2*ckin1&
               *1.d0/2.d0*alphas(lam_QCD5,mh,5.d0)/pi            
            
       end function
!------------------------------------------------------------------
       subroutine decay_hZZstar_sm(vhZZstar,vhZnunu_TB,vhZee_TB,vhZuu_TB,vhZdd_TB,OREW,ORQCD)
       double precision, intent(out):: vhZZstar,vhZnunu_TB,vhZee_TB,vhZuu_TB,vhZdd_TB
       integer, intent(in)::OREW,ORQCD
       double precision:: vhZnunu,vhZee,vhZuu,vhZdd
       double precision,parameter:: epsi=0.0001d0,mm=0.1d0
       vhZZstar=0.d0;vhZnunu_TB=0.d0;vhZee_TB=0.d0;vhZuu_TB=0.d0;vhZdd_TB=0.d0
       vhZnunu=0.d0;vhZee=0.d0;vhZuu=0.d0;vhZdd=0.d0
![EW=NLO, QCD=NNLO] ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         if (OREW .eq. 1 .and. ORQCD .eq.2) then 
         call romberg(vhZnunu,dGam_dqsq_hZneuneu_SM_NLOEW,epsi,(mh-mZ)**2-epsi)
         call romberg2d(vhZnunu_TB,dGam_dqsqdcos_hZneuneu_SM_NLOEW,epsi,(mh-mZ)**2-epsi&
         ,cos_the_i,cos_the_f) ! h->Znunu 
         vhZnunu=vhZnunu + vhZnunu_TB

         call romberg(vhZee,dGam_dqsq_hZee_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi)
         call romberg2d(vhZee_TB,dGam_dqsqdcos_hZee_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi&
         ,cos_the_i,cos_the_f) ! h->Zee  
         vhZee = vhZee+vhZee_TB

         call romberg(vhZuu,dGam_dqsq_hZuu_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi)
         call romberg2d(vhZuu_TB,dGam_dqsqdcos_hZuu_SM_NLOEWQCD,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi&
         ,cos_the_i,cos_the_f) ! h->Zuu  
         vhZuu=vhZuu+vhZuu_TB

         call romberg(vhZdd,dGam_dqsq_hZdd_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi)
         call romberg2d(vhZdd_TB,dGam_dqsqdcos_hZdd_SM_NLOEWQCD,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi&
         ,cos_the_i,cos_the_f) ! h->Zdd  
         vhZdd = vhZdd+ vhZdd_TB
 
         vhZZstar=3.d0*vhZnunu+3.d0*vhZee +6.d0*vhZuu+9.d0*vhZdd



![EW=NLO, QCD=NLO] ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         else if (OREW .eq. 1 .and. ORQCD .eq.1) then 
         call romberg(vhZnunu,dGam_dqsq_hZneuneu_SM_NLOEW,epsi,(mh-mZ)**2-epsi)
         call romberg2d(vhZnunu_TB,dGam_dqsqdcos_hZneuneu_SM_NLOEW,epsi,(mh-mZ)**2-epsi&
         ,cos_the_i,cos_the_f) ! h->Znunu 
         vhZnunu=vhZnunu + vhZnunu_TB

         call romberg(vhZee,dGam_dqsq_hZee_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi)
         call romberg2d(vhZee_TB,dGam_dqsqdcos_hZee_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi&
         ,cos_the_i,cos_the_f) ! h->Zee  
         vhZee = vhZee+vhZee_TB

         call romberg(vhZuu,dGam_dqsq_hZuu_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi)
         call romberg2d(vhZuu_TB,dGam_dqsqdcos_hZuu_SM_NLOEWQCD,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi&
         ,cos_the_i,cos_the_f) ! h->Zuu  
         vhZuu=vhZuu+vhZuu_TB

         call romberg(vhZdd,dGam_dqsq_hZdd_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi)
         call romberg2d(vhZdd_TB,dGam_dqsqdcos_hZdd_SM_NLOEWQCD,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi&
         ,cos_the_i,cos_the_f) ! h->Zdd  
         vhZdd = vhZdd+ vhZdd_TB
 
         vhZZstar=3.d0*vhZnunu+3.d0*vhZee +6.d0*vhZuu+9.d0*vhZdd

![EW=NLO, QCD=LO] ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         else if (OREW .eq. 1 .and. ORQCD .ge.-1 .and. ORQCD .le.0 ) then 
         call romberg(vhZnunu,dGam_dqsq_hZneuneu_SM_NLOEW,epsi,(mh-mZ)**2-epsi)
         call romberg2d(vhZnunu_TB,dGam_dqsqdcos_hZneuneu_SM_NLOEW,epsi,(mh-mZ)**2-epsi&
         ,cos_the_i,cos_the_f) ! h->Znunu 
         vhZnunu=vhZnunu + vhZnunu_TB

         call romberg(vhZee,dGam_dqsq_hZee_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi)
         call romberg2d(vhZee_TB,dGam_dqsqdcos_hZee_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi&
         ,cos_the_i,cos_the_f) ! h->Zee  
         vhZee = vhZee+vhZee_TB

         call romberg(vhZuu,dGam_dqsq_hZuu_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi)
         call romberg2d(vhZuu_TB,dGam_dqsqdcos_hZuu_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi&
         ,cos_the_i,cos_the_f) ! h->Zuu  
         vhZuu=vhZuu+vhZuu_TB

         call romberg(vhZdd,dGam_dqsq_hZdd_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi)
         call romberg2d(vhZdd_TB,dGam_dqsqdcos_hZdd_SM_NLOEW,(2.d0*mm)**2+epsi,(mh-mZ)**2-epsi&
         ,cos_the_i,cos_the_f) ! h->Zdd  
         vhZdd = vhZdd+ vhZdd_TB
 
         vhZZstar=3.d0*vhZnunu+3.d0*vhZee +6.d0*vhZuu+9.d0*vhZdd

![EW=LO, QCD=NNLO] ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         else if (OREW .eq. 0 .and. ORQCD .eq.2) then 

         call romberg(vhZnunu,dGam_dqsq_hZneuneu_SM_LO,epsi,(mh-mZ)**2-epsi)! h->Znunu 
         call romberg(vhZee,dGam_dqsq_hZee_SM_LO,0.d0+epsi,(mh-mZ)**2-epsi)! h->Zee  
         call romberg(vhZuu,dGam_dqsq_hZuu_SM_NLOQCD,0.d0+epsi,(mh-mZ)**2-epsi)! h->Zuu  
         call romberg(vhZdd,dGam_dqsq_hZdd_SM_NLOQCD,0.d0+epsi,(mh-mZ)**2-epsi)! h->Zdd  

         vhZZstar=3.d0*vhZnunu+3.d0*vhZee +6.d0*vhZuu+9.d0*vhZdd


![EW=LO, QCD=NLO] ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         else if (OREW .eq. 0 .and. ORQCD .eq.1) then 

         call romberg(vhZnunu,dGam_dqsq_hZneuneu_SM_LO,0.d0+epsi,(mh-mZ)**2-epsi)! h->Znunu 
         call romberg(vhZee,dGam_dqsq_hZee_SM_LO,0.d0+epsi,(mh-mZ)**2-epsi)! h->Zee  
         call romberg(vhZuu,dGam_dqsq_hZuu_SM_NLOQCD,0.d0+epsi,(mh-mZ)**2-epsi)! h->Zuu  
         call romberg(vhZdd,dGam_dqsq_hZdd_SM_NLOQCD,0.d0+epsi,(mh-mZ)**2-epsi)! h->Zdd  


         vhZZstar=3.d0*vhZnunu+3.d0*vhZee +6.d0*vhZuu+9.d0*vhZdd


![EW=LO, QCD=LO] ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        else if (OREW .eq. 0 .and. ORQCD .ge.-1 .and. ORQCD .le.0 ) then 
        vhZZstar = decay_hZZstar_tree(mh)
        else
        vhZZstar = 0.d0
        end if
 !         print*,"vhZnunu",vhZnunu
 !         print*,"vhZee",vhZee
 !         print*,"vhZuu",vhZuu
 !         print*,"vhZdd",vhZdd
 !         print*,"vhZZstar",vhZZstar

        end subroutine


!------------------ dGamma( h-->Wffp )/dqsq  at NLO ------------------

       double precision function dGam_dqsq_hWW_SM(qsq,mf,OR)
       double precision, intent(in):: qsq,mf
       character(*), intent(in):: OR
       double precision:: epsW,epsQ,x1,ckin_a,ckin_b
       epsW=mW**2/mh**2
       epsQ=qsq/mh**2
       x1=1.d0+epsW-epsQ
       ckin_a=sqrt(x1**2-4.d0*epsW)*((x1-6.d0*epsW)**2+8.d0*epsW*(1.d0-3.d0*epsW))
       ckin_b=(x1**2-4.d0*epsW)**(3.d0/2.d0)*(x1-2.d0*epsW)*mh**2/2.d0

       if (OR .eq. 'NLOEW') then ! LO + NLO EW
      ! note: this calcration is not included contributuion from hff vertex corrections and box diagrams as well as soft photon corrections
       dGam_dqsq_hWW_SM= &
           mh**3*(2.d0*mW**2/vev)**2/384.d0/pi**3/vev**2/(qsq-mW**2)**2*ckin_a*(&
           1.d0/2.d0&                                                                 !Tree cont.
               -(Del_r_SM())&                                                     !Delta_r
                -1.d0/2.d0*real(rdPiWW_SM(mW**2))&                                    !rdPiWW
                +1.d0/(2.d0*mW**2/vev)*(&
                         +real(Gam1_hWW_loop_SM(mW**2,qsq,mh**2))&                    !hWW Gam1 NLO cont.
                         +ckin_b/ckin_a*real(rGam2_hWW_SM(mW**2,qsq,mh**2))/mW**2 )&  !hWW Gam2 NLO cont.
                -real(rPiWW_SM(qsq))/(qsq-mW**2)&                                     !PiWW cont.
                -sqrt(2.d0)/g*real(rGam_Wffp_loop_SM(qsq,mf)) &                       !Wff NLO cont.
             )

      else if (OR .eq. 'NLOQCD') then ! only NLO QCD
       dGam_dqsq_hWW_SM= &
           mh**3*(2.d0*mW**2/vev)**2/384.d0/pi**3/vev**2/(qsq-mW**2)**2*ckin_a*&
           1.d0/2.d0*(1.d0+alphas(lam_QCD4,mh,4.d0)/pi)                                                                    

      else if (OR .eq. 'LO') then
       dGam_dqsq_hWW_SM= &
           mh**3*(2.d0*mW**2/vev)**2/384.d0/pi**3/vev**2/(qsq-mW**2)**2*ckin_a*&
           1.d0/2.d0                                                              
       else
       dGam_dqsq_hWW_SM= 0.d0
       end if

       end function
!------------------ dGamma( h-->Wffp )/dqsqdcos  at NLO ------------------
       double precision function dGam_dqsqdcos_hWW_SM_NLO(qsq,cos_the,mf)
       double precision, intent(in):: qsq,cos_the,mf
       double precision:: epsW,epsQ,x1,x2,ckin1,ckin2,ckin3,s24,s23,vf,vfp,Qf,Qfp,af,afp
! Be careful, if fermion masses digenerate, the bellow if sentence does not work. 
        if (mf .eq. mc) then 
        vf = vt ; af = at ; Qf = Qt
        vfp = vb ; afp = ab ; Qfp = Qb
        elseif (mf .eq. msir) then 
        vf = vb ; af = ab ; Qf = Qb
        vfp  = vt ; afp  = at ; Qfp  = Qt
        elseif (mf .eq. mtau) then 
        vf = ve ; af = ae ; Qf = Qe
        vfp = vnu ; afp = anu ; Qfp = 0.d0
        elseif (mf .eq. mmu) then 
        vf = ve ; af = ae ; Qf = Qe 
        vfp = vnu ; afp = anu ; Qfp = 0.d0
        endif
       epsW=mW**2/mh**2
       epsQ=qsq/mh**2
       x1=1.d0+epsW-epsQ
       x2=1.d0/2.d0*(2.d0-x1+sqrt(x1**2-4.d0*mW**2/mh**2)*cos_the)
       s24=mh**2*(1.d0-x2)
       s23=mh**2+mW**2-qsq-s24
       ckin1=4.d0*qsq+2.d0/mW**2*(s24*s23-mh**2*mW**2)
       ckin2=1.d0/mW**2*(s24*s23-mh**2*mW**2)*(mh**2-mW**2-qsq)
       ckin3=2.d0*qsq*(s24-s23)


       dGam_dqsqdcos_hWW_SM_NLO= &
             1.d0/(256.d0*pi**3*mh)*2.d0*g**2*(mW**2/vev)**2&
             *(1.d0/2.d0)*sqrt(x1**2-4.d0*epsW)/(qsq-mW**2)**2*ckin1*(&
              +(qsq-mW**2)*Thff_W_SM(vf,af,qsq,cos_the)&             ! hff NLO cont.  
              +(qsq-mW**2)*B_W_box(qsq,cos_the,mf)/ckin1&            ! Box NLO cont.     
              +1.d0/2.d0*Del_soft_Wffp(qsq,cos_the,mf,0.1d0)&        ! soft photon contriburion  ! final valuables in the argument is delE 
                )



! Full expressions
!        dGam_dqsqdcos_hWW_SM_NLO= 1.d0/(256.d0*pi**3*mh)*&
!             2.d0*g**2*(mW**2/vev)**2&
!             *(1.d0/2.d0)*sqrt(x1**2-4.d0*epsW)/(qsq-mW**2)**2*ckin1*(&
!                 1.d0/2.d0&                                                  !Tree cont.
!                        -real(Del_r_SM())&                                         !Delta_r
!                        -1.d0/2.d0*real(rdPiWW_SM(mW**2))&                         !rdPiWW
!                          +1.d0/(2.d0*mW**2/vev)*(&
!                          +real(Gam1_hWW_loop_SM(mW**2,qsq,mh**2))&                !hWW Gam1 NLO cont.
!                          +ckin2/ckin1*real(rGam2_hWW_SM(mW**2,qsq,mh**2))/mW**2&  !hWW Gam2 NLO cont.
!                          +ckin3/ckin1*real(rGam3_hWW_SM(mW**2,qsq,mh**2))/mW**2)& !hWW Gam3 NLO cont.
!                          -real(rPiWW_SM(qsq))/(qsq-mW**2)&                        ! PiWW cont.
!                          -sqrt(2.d0)/g*real(rGam_Wffp_loop_SM(qsq,mf))&
!                     +(qsq-mW**2)*Thff_W_SM(vf,af,qsq,cos_the)&             ! hff NLO cont.  
!                    +(qsq-mW**2)*B_W_box(qsq,cos_the,mf)/ckin1&         ! Box NLO cont.  
! !   ! soft photon contriburion               
!                   +1.d0/2.d0*Del_soft_Wffp(qsq,cos_the,mf,0.1d0)& ! final valuables in the argument is delE 
!                 )
       end function
!dqsq ------------------------------------------------------------------
! [LO] -------------------------------------------------
       double precision function dGam_dqsq_hWnutau_SM_LO(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hWnutau_SM_LO= dGam_dqsq_hWW_SM(qsq,mtau,'LO')
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsq_hWmunu_SM_LO(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hWmunu_SM_LO= dGam_dqsq_hWW_SM(qsq,mmu,'LO')
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsq_hWcs_SM_LO(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hWcs_SM_LO= 3.d0*dGam_dqsq_hWW_SM(qsq,mc,'LO') ! 3 is color factor
       end function

! [LO +part of NLO EW] -------------------------------------------------
       double precision function dGam_dqsq_hWnutau_SM_NLOEW(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hWnutau_SM_NLOEW= dGam_dqsq_hWW_SM(qsq,mtau,'NLOEW')
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsq_hWmunu_SM_NLOEW(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hWmunu_SM_NLOEW= dGam_dqsq_hWW_SM(qsq,mmu,'NLOEW')
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsq_hWcs_SM_NLOEW(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hWcs_SM_NLOEW= 3.d0*dGam_dqsq_hWW_SM(qsq,mc,'NLOEW') ! 3 is color factor
       end function
! [LO +part of NLO QCD] -------------------------------------------------
       double precision function dGam_dqsq_hWcs_SM_NLOQCD(qsq)
       double precision, intent(in):: qsq
       dGam_dqsq_hWcs_SM_NLOQCD= 3.d0*dGam_dqsq_hWW_SM(qsq,mc,'NLOQCD') ! 3 is color factor
       end function

!dqsqdcos  ------------------------------------------------------------------
! [part of NLO EW] ----------------------------------------------------------
       double precision function dGam_dqsqdcos_hWtaunu_SM_NLOEW(qsq,cos_the)
       double precision, intent(in):: qsq,cos_the

       dGam_dqsqdcos_hWtaunu_SM_NLOEW= &
       dGam_dqsqdcos_hWW_SM_NLO(qsq,cos_the,mtau)
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsqdcos_hWmunu_SM_NLOEW(qsq,cos_the)
       double precision, intent(in):: qsq,cos_the

       dGam_dqsqdcos_hWmunu_SM_NLOEW= &
       dGam_dqsqdcos_hWW_SM_NLO(qsq,cos_the,mmu)
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsqdcos_hWcs_SM_NLOEW(qsq,cos_the)
       double precision, intent(in):: qsq,cos_the
       double precision:: epsW,epsQ,x1,x2,ckin1,s24,s23

       dGam_dqsqdcos_hWcs_SM_NLOEW= &
       3.d0*dGam_dqsqdcos_hWW_SM_NLO(qsq,cos_the,mc) ! facotor 3 is color factor!!!
       end function

! [part of NLO EW+ NLO WCD] -------------------------------------------------
       double precision function dGam_dqsqdcos_hWcs_SM_NLOEWQCD(qsq,cos_the)
       double precision, intent(in):: qsq,cos_the
       double precision:: epsW,epsQ,x1,x2,ckin1,s24,s23
       epsW=mW**2/mh**2;epsQ=qsq/mh**2
       x1=1.d0+epsW-epsQ;x2=1.d0/2.d0*(2.d0-x1+sqrt(x1**2-4.d0*mW**2/mh**2)*cos_the)
       s24=mh**2*(1.d0-x2) ; s23=mh**2+mW**2-qsq-s24
       ckin1=4.d0*qsq+2.d0/mW**2*(s24*s23-mh**2*mW**2)

       dGam_dqsqdcos_hWcs_SM_NLOEWQCD= &
       3.d0*dGam_dqsqdcos_hWW_SM_NLO(qsq,cos_the,mc)& ! facotor 3 is color factor!!!
        !NLO QCD correction
        +3.d0/(256.d0*pi**3*mh)*&                     ! 3.d0 is color factor
        2.d0*g**2*(mW**2/vev)**2*(1.d0/2.d0)*sqrt(x1**2-4.d0*epsW)/(qsq-mW**2)**2*ckin1*1.d0/2.d0&
        *alphas(lam_QCD4,mh,4.d0)/pi 
       end function
!------------------------------------------------------------------
       double precision function dGam_dqsqdcos_hWsc_SM_NLOEWQCD(qsq,cos_the)
       double precision, intent(in):: qsq,cos_the
       double precision:: epsW,epsQ,x1,x2,ckin1,s24,s23
        epsW=mW**2/mh**2 ; epsQ=qsq/mh**2
        x1=1.d0+epsW-epsQ; x2=1.d0/2.d0*(2.d0-x1+sqrt(x1**2-4.d0*mW**2/mh**2)*cos_the)
        s24=mh**2*(1.d0-x2); s23=mh**2+mW**2-qsq-s24
        ckin1=4.d0*qsq+2.d0/mW**2*(s24*s23-mh**2*mW**2)

       dGam_dqsqdcos_hWsc_SM_NLOEWQCD= &
       3.d0*dGam_dqsqdcos_hWW_SM_NLO(qsq,cos_the,msir) &   ! facotor 3 is color factor!!!
       !NLO QCD correction
        +3.d0/(256.d0*pi**3*mh)*&                          ! 3.d0 is color factor
        2.d0*g**2*(mW**2/vev)**2*(1.d0/2.d0)*sqrt(x1**2-4.d0*epsW)/(qsq-mW**2)**2*ckin1*1.d0/2.d0&
        *alphas(lam_QCD4,mh,4.d0)/pi 

       end function

!------------------------------------------------------------------
       subroutine decay_hWWstar_sm(vhWWstar,vhWcs_TBSH,vhWmunu_TBSH,OREW,ORQCD)
       double precision, intent(out):: vhWWstar,vhWcs_TBSH,vhWmunu_TBSH
       integer, intent(in)::OREW,ORQCD
       double precision:: vhWcs,vhWmunu
       double precision, parameter:: epsi=0.0001d0
       double precision, parameter:: epsics=0.1d0 !comment:if epsics = epsi,the cancellation of div parameter doesn't happen for h->Wcs
       vhWWstar=0.d0;vhWcs_TBSH=0.d0;vhWmunu_TBSH=0.d0
       vhWcs=0.d0;vhWmunu=0.d0

![EW=NLO, QCD=NNLO] ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
        if (OREW .eq. 1 .and. ORQCD .eq.2) then 

          call romberg(vhWcs,dGam_dqsq_hWcs_SM_NLOEW,(mc+msir)**2+epsics,(mh-mW)**2-epsics)
          call romberg2d(vhWcs_TBSH,dGam_dqsqdcos_hWcs_SM_NLOEWQCD,(mc+msir)**2+epsics,(mh-mW)**2-epsi&
          ,cos_the_i,cos_the_f) ! h->W-cs~
!          vhWcs_TBSH=vhWcs_TBSH+4.173d0*10**(-6.d0) ! hard photon in the case of deltaE= 0.1 mf=mc=1.275, mfp=ms=0.095 
          vhWcs_TBSH=vhWcs_TBSH+3.9092d0*10**(-6.d0) ! hard photon in the case of deltaE= 0.1 mf=mc=1.67, mfp=ms=0.095,mh =125.1 
          vhWcs= vhWcs+vhWcs_TBSH
        
          call romberg(vhWmunu,dGam_dqsq_hWmunu_SM_NLOEW,(2.d0*mmu)**2+epsi,(mh-mW)**2-epsi)
          call romberg2d(vhWmunu_TBSH,dGam_dqsqdcos_hWmunu_SM_NLOEW,(2.d0*mmu)**2+epsi,(mh-mW)**2-epsi&
          ,cos_the_i,cos_the_f) ! h->W+mu-n
!          vhWmunu_TBSH = vhWmunu_TBSH+4.2118d0*10**(-6.d0) ! hard photon contributions in the case of deltaE= 0.1 mf=mmu=0.1, mfp=mnu=0. 
          vhWmunu_TBSH = vhWmunu_TBSH+4.2263d0*10**(-6.d0) ! hard photon contributions in the case of deltaE= 0.1 mf=mmu=0.1056583745d0, mfp=mnu=0,mh=125.1
          vhWmunu=vhWmunu+vhWmunu_TBSH

          vhWWstar =  2.d0*(3.d0*vhWmunu+2.d0*vhWcs)


![EW=NLO, QCD=NLO] ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
        else if (OREW .eq. 1 .and. ORQCD .eq.1) then 

          call romberg(vhWcs,dGam_dqsq_hWcs_SM_NLOEW,(mc+msir)**2+epsics,(mh-mW)**2-epsi)
          call romberg2d(vhWcs_TBSH,dGam_dqsqdcos_hWcs_SM_NLOEWQCD,(mc+msir)**2+epsics,(mh-mW)**2-epsi&
          ,cos_the_i,cos_the_f) ! h->W-cs~
!          vhWcs_TBSH=vhWcs_TBSH+4.173d0*10**(-6.d0) ! hard photon in the case of deltaE= 0.1 mf=mc=1.275, mfp=ms=0.095 
          vhWcs_TBSH=vhWcs_TBSH+3.9092d0*10**(-6.d0) ! hard photon in the case of deltaE= 0.1 mf=mc=1.67, mfp=ms=0.095,mh =125.1 
          vhWcs= vhWcs+vhWcs_TBSH
        
          call romberg(vhWmunu,dGam_dqsq_hWmunu_SM_NLOEW,(2.d0*mmu)**2+epsi,(mh-mW)**2-epsi)
          call romberg2d(vhWmunu_TBSH,dGam_dqsqdcos_hWmunu_SM_NLOEW,(2.d0*mmu)**2+epsi,(mh-mW)**2-epsi&
          ,cos_the_i,cos_the_f) ! h->W+mu-n
!          vhWmunu_TBSH = vhWmunu_TBSH+4.2118d0*10**(-6.d0) ! hard photon contributions in the case of deltaE= 0.1 mf=mmu=0.1, mfp=mnu=0. 
          vhWmunu_TBSH = vhWmunu_TBSH+4.2263d0*10**(-6.d0) ! hard photon contributions in the case of deltaE= 0.1 mf=mmu=0.1056583745d0, mfp=mnu=0,mh=125.1
          vhWmunu=vhWmunu+vhWmunu_TBSH

          vhWWstar =  2.d0*(3.d0*vhWmunu+2.d0*vhWcs)

![EW=NLO, QCD=LO] ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
        else if (OREW .eq. 1 .and. ORQCD .ge.-1 .and. ORQCD .le.0 ) then
          call romberg(vhWcs,dGam_dqsq_hWcs_SM_NLOEW,(mc+msir)**2+epsics,(mh-mW)**2-epsi)
          call romberg2d(vhWcs_TBSH,dGam_dqsqdcos_hWcs_SM_NLOEW,(mc+msir)**2+epsics,(mh-mW)**2-epsi&
          ,cos_the_i,cos_the_f) ! h->W-cs~
!          vhWcs_TBSH=vhWcs_TBSH+4.173d0*10**(-6.d0) ! hard photon in the case of deltaE= 0.1 mf=mc=1.275, mfp=ms=0.095 
          vhWcs_TBSH=vhWcs_TBSH+3.9092d0*10**(-6.d0) ! hard photon in the case of deltaE= 0.1 mf=mc=1.67, mfp=ms=0.095,mh =125.1 
          vhWcs= vhWcs+vhWcs_TBSH
        
          call romberg(vhWmunu,dGam_dqsq_hWmunu_SM_NLOEW,(2.d0*mmu)**2+epsi,(mh-mW)**2-epsi)
          call romberg2d(vhWmunu_TBSH,dGam_dqsqdcos_hWmunu_SM_NLOEW,(2.d0*mmu)**2+epsi,(mh-mW)**2-epsi&
          ,cos_the_i,cos_the_f) ! h->W+mu-n
!          vhWmunu_TBSH = vhWmunu_TBSH+4.2118d0*10**(-6.d0) ! hard photon contributions in the case of deltaE= 0.1 mf=mmu=0.1, mfp=mnu=0. 
          vhWmunu_TBSH = vhWmunu_TBSH+4.2263d0*10**(-6.d0) ! hard photon contributions in the case of deltaE= 0.1 mf=mmu=0.1056583745d0, mfp=mnu=0,mh=125.1
          vhWmunu=vhWmunu+vhWmunu_TBSH

          vhWWstar =  2.d0*(3.d0*vhWmunu+2.d0*vhWcs)

![EW=LO, QCD=NLO] ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
        else if (OREW .eq. 0 .and. ORQCD .eq.2) then 

          call romberg(vhWcs,dGam_dqsq_hWcs_SM_NLOQCD,0.d0+epsi,(mh-mW)**2-epsi)! h->W-cs~
          call romberg(vhWmunu,dGam_dqsq_hWmunu_SM_LO,0.d0+epsi,(mh-mW)**2-epsi)! h->W+mu-n

          vhWWstar =  2.d0*(3.d0*vhWmunu+2.d0*vhWcs)


![EW=LO, QCD=NLO] ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
        else if (OREW .eq. 0 .and. ORQCD .eq.1) then 

          call romberg(vhWcs,dGam_dqsq_hWcs_SM_NLOQCD,0.d0+epsi,(mh-mW)**2-epsi)! h->W-cs~
          call romberg(vhWmunu,dGam_dqsq_hWmunu_SM_LO,0.d0+epsi,(mh-mW)**2-epsi)! h->W+mu-n


          vhWWstar =  2.d0*(3.d0*vhWmunu+2.d0*vhWcs)


![EW=NLO, QCD=NLO] ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
        else if (OREW .eq. 0 .and. ORQCD .ge.-1 .and. ORQCD .le.0 ) then
        vhWWstar = decay_hWWstar_tree(mh)
        else
        vhWWstar = 0.d0
        end if
!          print*,'vhWcs',vhWcs
!          print*,'vhWmunu',vhWmunu
!          print*,'vhWWstar',vhWWstar
        end subroutine

!------------------------------------------------------------------
       double precision function Higgs_width_SM_LO(mh)
       double precision, intent(in):: mh

       Higgs_width_SM_LO= &
        decay_hbb_sm(mh,0,0)+decay_htautau_sm(mh,0)+decay_hcc_sm(mh,0,0)&
        +decay_hZZstar_tree(mh)+decay_hWWstar_tree(mh)&
        +decay_hZgam_sm(mh,0)+decay_hgamgam_sm(mh,0)+decay_hgg_sm(mh,0)

       end function
!------------------------------------------------------------------
       subroutine Switch_HOEW(OREW,S_NLOEW)   ! For EW correction
       integer, intent(in)::OREW
       double precision, intent(out)::S_NLOEW
       if (OREW .eq. 0 ) then
          S_NLOEW=0.d0
       else if (OREW .eq. 1) then
          S_NLOEW=1.d0
       else
          S_NLOEW=-1d10
       end if
       end subroutine
!------------------------------------------------------------------
       subroutine Switch_HOQCD(ORQCD,S_NLOQCD,S_NNLOQCD,S_RMSM) ! For QCD correction
       integer, intent(in)::ORQCD
       double precision, intent(out)::S_NLOQCD,S_NNLOQCD,S_RMSM
       if (ORQCD .eq. -1 ) then ! LO (qurk mass: OS)
          S_NLOQCD=0.d0 ; S_NNLOQCD=0.d0 ; S_RMSM=0.d0
       else if(ORQCD .eq. 0 ) then ! LO (qurk mass: MSbar)
           S_NLOQCD=0.d0 ; S_NNLOQCD=0.d0 ; S_RMSM=1.d0
       else if (ORQCD .eq. 1) then ! NLO QCD
          S_NLOQCD=1.d0 ; S_NNLOQCD=0.d0 ; S_RMSM=1.d0
       else if (ORQCD .eq. 2) then ! NNLO QCD
          S_NLOQCD=1.d0 ; S_NNLOQCD=1.d0 ; S_RMSM=1.d0
       else
          S_NLOQCD=-1d10 ; S_NNLOQCD=-1d10 ; S_RMSM=-1d10
       end if
       end subroutine

!##########################################################
!
!    SECTION VII OTHER QUANTITIES
!
!##########################################################
!------------- renormalized PiWW function    -------
       complex (kind(0d0)) function rPiWW_SM(qsq)
         double precision, intent(in):: qsq

        rPiWW_SM = PiWW_F(qsq) + PiWW_B_SM(qsq)- PiWW_pinch(qsq) - del_mWsq_SM() + (qsq-mW**2)*del_ZW_SM()


         end function
!----------- renormalized dPiZZ function ----------
       complex (kind(0d0)) function rdPiZZ_SM(qsq)
         double precision, intent(in):: qsq

        rdPiZZ_SM = dPiZZ_F(qsq)+dPiZZ_B_SM(qsq)+ del_ZZ_SM() &
                  -dPiZZ_pinch(qsq) 

         end function
!----------- renormalized dPiWW function ----------
       complex (kind(0d0)) function rdPiWW_SM(qsq)
         double precision, intent(in):: qsq

        rdPiWW_SM = dPiWW_F(qsq)+dPiWW_B_SM(qsq)+ del_ZW_SM() &
                  -dPiWW_pinch(qsq) 

         end function
!-------------- Delta_r  -------------------
         double precision function Del_r_SM()


        Del_r_SM = rPiWW_SM(0.d0)/mW**2 + alpha_em/(4.d0*pi*sw**2)*(6.d0 + (7.d0-4.d0*sw**2)/(2.d0*sw**2)*log(cw**2))

         end function
!------------- renormalized ZZ function --------------------
       complex (kind(0d0)) function rPiZZ_SM(qsq)
       double precision, intent(in):: qsq

       rPiZZ_SM= PiZZ_F(qsq)+PiZZ_B_SM(qsq)-PiZZ_pinch(qsq)&
            -del_mZsq_SM()+del_ZZ_SM()*(qsq-mZ**2)
       end function
!------------------- renormalized Zgam function  -------------
       complex (kind(0d0)) function rPiZG_SM(qsq)
       double precision, intent(in):: qsq

            rPiZG_SM= PiZG_F(qsq)+PiZG_B_SM(qsq)-PiZG_pinch(qsq)& 
                      -PiZG_F(0.d0)-PiZG_B_SM(0.d0)+PiZG_pinch(0.d0)&
                      +qsq*(del_swsq_SM()/(sw*cw)&
                      +2.d0/mZ**2*(PiZG_F(0.d0)+PiZG_B_SM(0.d0)-PiZG_pinch(0.d0)) )


       end function
!------------------- QED correction for h->ff  -------------[A. Dabelstein, W. Hollik, Z. Phys. C53, 507]
! for A_QED we have used Eq.(2.15) in [Phys. Rep. 457(2008)1]
       double precision function Del_QED_hff(mf)
       double precision:: mf,Qf
       double precision:: beta0,A_QED  
#include "looptools.h"
       if (mf .eq. mt) then 
        Qf=qt
       elseif (mf .eq. mc) then 
        Qf=qt
       elseif (mf .eq. mb) then 
        Qf=qb
       elseif (mf .eq. mtau) then 
        Qf=qe
        elseif (mf .eq. mmu) then 
        Qf = qe 
        endif

       beta0 = sqrt(1.d0-4.d0*mf**2/mh**2)
       A_QED = (1.d0+beta0**2)*(&
            4.d0*r8_spence((1.d0-beta0)/(1.d0+beta0))+2.d0*r8_spence(-(1.d0-beta0)/(1.d0+beta0))&
            -3.d0*log(2.d0/(1.d0+beta0))*log((1.d0+beta0)/(1.d0-beta0))&
            -2.d0*log(beta0)*log((1.d0+beta0)/(1.d0-beta0)) )&
!
             -3.d0*beta0*log(4.d0/(1.d0-beta0**2))-4.d0*beta0*log(beta0)
       Del_QED_hff = alpha_em/pi*Qf**2*( A_QED/beta0&
                   +(3.d0+34.d0*beta0**2-13.d0*beta0**4)/16.d0/beta0**3*log((1.d0+beta0)/(1.d0-beta0))&
                   +3.d0*(-1.d0+7.d0*beta0**2)/8.d0/beta0**2 &
                   )
       end function

!------------------- soft photon correction for h->Wffp  -------------
       double precision function I_soft_Wffp(qsq,cos_the,mf,delE,k)
       double precision, intent(in):: qsq,mf,cos_the,delE
       double precision:: mfp,Qf,Qfp,E2,absp2,s23,s24,q340,beta2,betabar,gambar
       double precision:: E3hat,E4hat,beta34,p3p4,p3p4_2,p2p4,p2p3
       double precision:: p1(4),p2(4),p3(4),p4(4),alpha34bar,alpha23bar,alpha24bar
       double precision:: absp3,absp4,v23,v24,v34
       double precision:: mgam
       integer, intent(in)::k
#include "looptools.h"
        mgam = mphoton

        if (mf .eq. mc) then 
        Qf = Qt ; Qfp = Qb ; mfp =msir
        elseif (mf .eq. msir) then 
        Qf = Qb ; Qfp  = Qt ; mfp = mc
        elseif (mf .eq. mtau) then 
        Qf = Qe ; Qfp = 0.d0 ; mfp=0.d0
        elseif (mf .eq. mmu) then 
        Qf = Qe ; Qfp = 0.d0 ; mfp=0.d0
        endif

       q340=mh/2.d0*(1.d0-mW**2/mh**2+qsq/mh**2)
       beta2=sqrt(1.d0-2.d0*(mW**2/mh**2+qsq/mh**2)+(mW**2/mh**2-qsq/mh**2)**2)
       betabar=-mh/(2.d0*q340)*beta2
       gambar=q340/sqrt(qsq)
       beta34=sqrt(1.d0-2.d0*(mf**2/qsq+mfp**2/qsq)+(mf**2/qsq-mfp**2/qsq)**2)
       absp2=mh/2.d0*beta2

       E2=mh/2.d0*(1.d0+mW**2/mh**2-qsq/mh**2)
       E3hat= sqrt(qsq)/2.d0*(1.d0+mf**2/qsq-mfp**2/qsq)
       E4hat= sqrt(qsq)/2.d0*(1.d0-mf**2/qsq+mfp**2/qsq)

       p1(1) = mh; p1(2) = 0.d0; p1(3) = 0.d0; p1(4) = 0.d0;
       p2(1) = E2; p2(2) = 0.d0; p2(3) = 0.d0; p2(4) = absp2;
       p3(1) = E3hat*gambar+1.d0/2.d0*sqrt(qsq)*beta34*betabar*gambar*cos_the
       p3(2) = 1.d0/2.d0*sqrt(qsq)*beta34*sqrt(1.d0-cos_the**2)
       p3(3) = 0.d0
       p3(4) = E3hat*betabar*gambar+1.d0/2.d0*sqrt(qsq)*beta34*gambar*cos_the
       p4(1) = E4hat*gambar-1.d0/2.d0*sqrt(qsq)*beta34*betabar*gambar*cos_the
       p4(2) = -1.d0/2.d0*sqrt(qsq)*beta34*sqrt(1.d0-cos_the**2)
       p4(3) = 0.d0
       p4(4) = E4hat*betabar*gambar-1.d0/2.d0*sqrt(qsq)*beta34*gambar*cos_the

       p3p4 = p3(1)*p4(1)-p3(2)*p4(2)-p3(3)*p4(3)-p3(4)*p4(4)
       p2p3 = p3(1)*p2(1)-p3(2)*p2(2)-p3(3)*p2(3)-p3(4)*p2(4)
       p2p4 = p4(1)*p2(1)-p4(2)*p2(2)-p4(3)*p2(3)-p4(4)*p2(4)
       s23 = mW**2+2.d0*p2p3+mf**2 
       s24 = mW**2+2.d0*p2p4+mfp**2
 
       alpha34bar=p3p4+sqrt(p3p4**2-mf**2*mfp**2)
       alpha23bar=p2p3+sqrt(p2p3**2-mf**2*mfp**2)
       alpha24bar=p2p4+sqrt(p2p4**2-mf**2*mfp**2)
       
       absp3=sqrt(p3(1)**2-mf**2); absp4=sqrt(p4(1)**2-mfp**2)
       v34=(alpha34bar**2-mfp**2*mf**2)/(alpha34bar*p4(1)-mfp**2*p3(1))/2.d0
       v23=(alpha23bar**2-mW**2*mf**2)/(alpha23bar*p2(1)-mW**2*p3(1))/2.d0
       v24=(alpha24bar**2-mW**2*mfp**2)/(alpha24bar*p2(1)-mW**2*p4(1))/2.d0


       If( k.eq.22) then
       I_soft_Wffp= 2.d0*pi*(log(4.d0*delE**2/mgam**2)+E2/absp2*log(mW**2/(E2+absp2)**2) )

       else If(k.eq.33) then
       I_soft_Wffp= 2.d0*pi*(log(4.d0*delE**2/mgam**2)+p3(1)/absp3*log(mf**2/(p3(1)+absp3)**2) )



       else If(k.eq.44) then
       I_soft_Wffp= 2.d0*pi*(log(4.d0*delE**2/mgam**2)+p4(1)/absp4*log(mfp**2/(p4(1)+absp4)**2) )

       else If(k.eq.34) then
       I_soft_Wffp= -2.d0*pi*(alpha34bar**2+mf**2*mfp**2)/(alpha34bar**2-mf**2*mfp**2)*log(4.d0*delE**2/mgam**2)&
            *1.d0/2.d0*log(mf**2/alpha34bar*mfp**2/alpha34bar)&
!             !finite term
             +2.d0*pi*(alpha34bar**2+mf**2*mfp**2)/(alpha34bar**2-mf**2*mfp**2)*(&
             +1.d0/4.d0*(log( mfp**2/(p4(1)+absp4)**2 ) )**2-1.d0/4.d0*( log(mf**2/(p3(1)+absp3)**2 ) )**2&
             +r8_spence(1.d0-alpha34bar/mfp**2*(p4(1)-absp4)/v34)+r8_spence(1.d0-alpha34bar/mfp**2*(p4(1)+absp4)/v34)&    
             -r8_spence(1.d0-(p3(1)-absp3)/v34)-r8_spence(1.d0-(p3(1)+absp3)/v34)&    
             )

       else If(k.eq.23) then
       I_soft_Wffp= -2.d0*pi*(alpha23bar**2+mf**2*mW**2)/(alpha23bar**2-mf**2*mW**2)*log(4.d0*delE**2/mgam**2)&
            *1.d0/2.d0*log(mf**2/alpha23bar*mW**2/alpha23bar)&
!             !finite term
            +2.d0*pi*(alpha23bar**2+mf**2*mW**2)/(alpha23bar**2-mf**2*mW**2)*(&
            +1.d0/4.d0*(log(mW**2/(E2+absp2)**2) )**2-1.d0/4.d0*(log(mf**2/(p3(1)+absp3)**2 ) )**2&
            +r8_spence(1.d0-alpha23bar/mW**2*(E2-absp2)/v23 )+r8_spence(1.d0-alpha23bar/mW**2*(E2+absp2)/v23 )&    
            -r8_spence(1.d0-(p3(1)-absp3)/v23 )-r8_spence(1.d0-(p3(1)+absp3)/v23 )&
              )

       else If(k.eq.24) then
       I_soft_Wffp= -2.d0*pi*(alpha24bar**2+mfp**2*mW**2)/(alpha24bar**2-mfp**2*mW**2)*log(4.d0*delE**2/mgam**2)&
            *1.d0/2.d0*log(mfp**2/alpha24bar*mW**2/alpha24bar)&
!             !finite term
            +2.d0*pi*(alpha24bar**2+mfp**2*mW**2)/(alpha24bar**2-mfp**2*mW**2)*(&
            +1.d0/4.d0*(log(mW**2/(E2+absp2)**2) )**2-1.d0/4.d0*(log(mfp**2/(p4(1)+absp4)**2 ) )**2&
            +r8_spence(1.d0-alpha24bar/mW**2*(E2-absp2)/v24 )+r8_spence(1.d0-alpha24bar/mW**2*(E2+absp2)/v24 )&    
            -r8_spence(1.d0-(p4(1)-absp4)/v24 )-r8_spence(1.d0-(p4(1)+absp4)/v24 )&
              )

       end if


       end function
!---------------------------------------------
       double precision function Del_soft_Wffp(qsq,cos_the,mf,delE)
       double precision, intent(in):: qsq,mf,cos_the,delE
       double precision:: mfp,Qf,Qfp,E2,E3,E4,absp2,s23,s24,q340,beta2,betabar


        if (mf .eq. mc) then 
        Qf = Qt ; Qfp = Qb ; mfp =msir
       Del_soft_Wffp = alpha_em/(2.d0*pi)**2*(&
       -Qf**2*I_soft_Wffp(qsq,cos_the,mf,delE,33)&
       -Qfp**2*I_soft_Wffp(qsq,cos_the,mf,delE,44)&
       -I_soft_Wffp(qsq,cos_the,mf,delE,22)&
       +2.d0*Qf*Qfp*I_soft_Wffp(qsq,cos_the,mf,delE,34)&
       +2.d0*Qf*(Qf-Qfp)*I_soft_Wffp(qsq,cos_the,mf,delE,23)&
       -2.d0*Qfp*(Qf-Qfp)*I_soft_Wffp(qsq,cos_the,mf,delE,24)&
       )

        elseif (mf .eq. msir) then 
        Qf = Qb ; Qfp  = Qt ; mfp = mc
       Del_soft_Wffp = alpha_em/(2.d0*pi)**2*(&
       -Qf**2*I_soft_Wffp(qsq,cos_the,mf,delE,33)&
       -Qfp**2*I_soft_Wffp(qsq,cos_the,mf,delE,44)&
       -I_soft_Wffp(qsq,cos_the,mf,delE,22)&
       +2.d0*Qf*Qfp*I_soft_Wffp(qsq,cos_the,mf,delE,34)&
       +2.d0*Qf*(Qf-Qfp)*I_soft_Wffp(qsq,cos_the,mf,delE,23)&
       -2.d0*Qfp*(Qf-Qfp)*I_soft_Wffp(qsq,cos_the,mf,delE,24)&
       )

        elseif (mf .eq. mtau) then 
        Qf = Qe ; Qfp = 0.d0 ; mfp=0.d0
       Del_soft_Wffp = alpha_em/(2.d0*pi)**2*(&
       -Qf**2*I_soft_Wffp(qsq,cos_the,mf,delE,33)&
       -I_soft_Wffp(qsq,cos_the,mf,delE,22)&
       +2.d0*Qf*(Qf-Qfp)*I_soft_Wffp(qsq,cos_the,mf,delE,23)&
       )
        elseif (mf .eq. mmu) then 
        Qf = Qe ; Qfp = 0.d0 ; mfp=0.d0
       Del_soft_Wffp = alpha_em/(2.d0*pi)**2*(&
       -Qf**2*I_soft_Wffp(qsq,cos_the,mf,delE,33)&
       -I_soft_Wffp(qsq,cos_the,mf,delE,22)&
       +2.d0*Qf*(Qf-Qfp)*I_soft_Wffp(qsq,cos_the,mf,delE,23)&
       )

       else
          Del_soft_Wffp =0.d0
       endif

       end function

!------------------- soft photon correction for Z-->ff -------------
       double precision function Del_soft_Zff(qsq,cos_the,mf)
       double precision, intent(in):: qsq,mf,cos_the
       double precision:: mfp,Qf,Qfp,delE,E2,E3,E4,absp2,s23,s24,q340,beta2,betabar,beta34
       double precision:: getlambda,mgam,getmudim
       
        if (mf .eq. mc) then 
        Qf = Qt
        elseif (mf .eq. mb) then 
        Qf = Qb
        elseif (mf .eq. msir) then 
        Qf = Qb
        elseif (mf .eq. mtau) then 
        Qf = Qe
        elseif (mf .eq. mmu) then 
        Qf = Qe 
        endif


       delE =1.d0

       q340=mh/2.d0*(1.d0-mZ**2/mh**2+qsq/mh**2)
       beta2=sqrt(1.d0-2.d0*(mZ**2/mh**2+qsq/mh**2)+(mZ**2/mh**2-qsq/mh**2)**2)
       betabar=-mh/(2.d0*q340)*beta2
       beta34 = sqrt(1.d0-4.d0*mf**2/qsq)


! expressinos in nonzero mf,mfp 
        s24=1.d0/(2.d0*qsq)*(&
            (mf**2-mfp**2)*(mh**2-mZ**2)+(mf**2+mfp**2+mZ**2+mh**2)*qsq-qsq**2-cos_the&
            *sqrt(mh**4+(mZ**2-qsq)**2-2.d0*mh**2*(mZ**2+qsq))&
            *sqrt(mf**4+(mfp**2-qsq)**2-2.d0*mf**2*(mfp**2+qsq)) )
        s23=mh**2+mZ**2+mf**2+mfp**2-qsq-s24


       Del_soft_Zff =& 
!!divergent term        
          alpha_em*Qf**2/pi*(&
           (1.d0+beta34**2)/2.d0/beta34*log((1.d0+beta34)/(1.d0-beta34))&
            -1.d0 &
          )*log(4.d0*delE**2/getlambda())   &
!!finite term                              
          -alpha_em/2.d0/pi*(&
          Qf**2*(1.d0/2.d0*(log(mf**2/4.d0/E3**2))**2 +log(mf**2/4.d0/E3**2)+pi**2/3.d0&
          +1.d0/2.d0*(log(mf**2/4.d0/E4**2))**2+log(mf**2/4.d0/E4**2)+pi**2/3.d0 )&
           )

       end function
!------------------- virtual photon correction for Zff vertex -------------
       complex (kind(0d0)) function rGamV_Zff_photon_loop_SM(qsq,vf,af,Iff,Qf,mf)
       double precision, intent(in):: vf,af,Iff,qsq,Qf,mf
       double precision::getlambda

       rGamV_Zff_photon_loop_SM= &
            -alpha_em/(4.d0*pi)*Qf**2*vf*gz*(&
             Fa_Vff(mf**2,mf**2,qsq,mf,dsqrt(getlambda()),mf)&
             +2.d0*b1f(mf**2,mf,dsqrt(getlambda()))+1.d0&
       )
       end function
!---------------------------------
       complex (kind(0d0)) function rGamA_Zff_photon_loop_SM(qsq,vf,af,Iff,Qf,mf)
       double precision, intent(in):: vf,af,Iff,qsq,Qf,mf
       double precision::getlambda

       rGamA_Zff_photon_loop_SM= &
            alpha_em/(4.d0*pi)*Qf**2*af*gz*(&
             Fa_Vff(mf**2,mf**2,qsq,mf,dsqrt(getlambda()),mf)&
             +2.d0*b1f(mf**2,mf,dsqrt(getlambda()))+1.d0&
       )
       end function
!---------------------------------
       double precision function rmW_SM()
       rmW_SM=sqrt(&
              mZ**2/2.d0*(1.d0&
                  +sqrt(1.d0-4.d0*pi*alpha_em/&
                  (sqrt(2.d0)*G_f*mZ**2*(1.d0-Del_r_SM() ) &
                  ))& 
              ))
       end function


       end module HCOUP_SM
